@@ -1,17 +1,21 @@
-/*  $Id: radius.c 9250 2011-07-17 18:06:00Z iulius $
+/*  $Id: radius.c 9695 2014-09-20 05:53:22Z iulius $
 **
 **  Authenticate a user against a remote radius server.
 */
 
 #include "config.h"
 #include "clibrary.h"
-#include "portable/time.h"
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <netdb.h>
 #include <signal.h>
 
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
+
 /* Needed on AIX 4.1 to get fd_set and friends. */
 #if HAVE_SYS_SELECT_H
 # include <sys/select.h>
