@@ -1,4 +1,4 @@
-/*  $Id: batcher.c 8774 2009-11-15 09:23:27Z iulius $
+/*  $Id: batcher.c 9911 2015-07-04 21:32:56Z iulius $
 **
 **  Read batchfiles on standard input and spew out batches.
 */
@@ -54,7 +54,9 @@ BATCHstart(void)
     char	buff[SMBUF];
 
     if (Processor && *Processor) {
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
 	snprintf(buff, sizeof(buff), Processor, Host);
+#pragma GCC diagnostic warning "-Wformat-nonliteral"
 	F = popen(buff, "w");
 	if (F == NULL)
 	    return NULL;
@@ -370,7 +372,9 @@ main(int ac, char *av[])
     SendIt:
 	/* Now we can start to send the article! */
 	if (Separator && *Separator) {
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
 	    snprintf(buff, sizeof(buff), Separator, BytesInArt);
+#pragma GCC diagnostic warning "-Wformat-nonliteral"
 	    BytesInCB += strlen(buff) + 1;
 	    BytesWritten += strlen(buff) + 1;
 	    if (fprintf(F, "%s\n", buff) == EOF || ferror(F)) {
