@@ -1,4 +1,4 @@
-/*  $Id: buffchan.c 8418 2009-04-12 21:08:22Z iulius $
+/*  $Id: buffchan.c 9911 2015-07-04 21:32:56Z iulius $
 **
 **  Buffered file exploder for innd.
 */
@@ -179,7 +179,9 @@ SITEfind(char *Name, bool CanCreate)
 
     /* Fill in the structure for the new site. */
     sp->Name = xstrdup(Name);
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
     snprintf(buff, sizeof(buff), Format, Map ? MAPname(Name) : sp->Name);
+#pragma GCC diagnostic warning "-Wformat-nonliteral"
     sp->Filename = xstrdup(buff);
     if (BufferMode == 'u')
 	sp->Buffer = NULL;
@@ -379,7 +381,7 @@ main(int ac, char *av[])
 	case 'd':
 	    Directory = optarg;
 	    if (Format == NULL)
-		Format =xstrdup("%s");
+                Format = xstrdup("%s");
 	    break;
 	case 'f':
 	    Fields = atoi(optarg);
@@ -409,7 +411,6 @@ main(int ac, char *av[])
 	    break;
 	}
     ac -= optind;
-    av += optind;
     if (ac)
 	die("usage error");
 
