@@ -1,4 +1,4 @@
-/*  $Id: inndf.c 6677 2004-03-03 18:36:07Z hkehoe $
+/*  $Id: inndf.c 9891 2015-06-03 19:08:50Z iulius $
 **
 **  Reports free kilobytes (not disk blocks) or free inodes.
 **
@@ -48,10 +48,10 @@
 
 #include "inn/innconf.h"
 #include "inn/messages.h"
+#include "inn/overview.h"
 #include "inn/qio.h"
-#include "libinn.h"
-#include "ov.h"
-#include "paths.h"
+#include "inn/libinn.h"
+#include "inn/paths.h"
 
 /* The portability mess.  Hide everything in macros so that the actual code
    is relatively clean.  SysV uses statvfs, BSD uses statfs, and ULTRIX is
@@ -60,8 +60,8 @@
    df_declare declares a variable of the appropriate type to pass to df_stat
    along with a path; df_stat will return true on success, false on failure.
    df_avail gives the number of free blocks, the size of those blocks given
-   in df_bsize (which handles SysV's weird fragment vs. preferred block size
-   thing).  df_inodes returns the free inodes. */
+   in df_scale (which handles SysV's weird fragment vs. preferred block size
+   thing).  df_favail returns the free inodes. */
 #if HAVE_STATVFS
 # include <sys/statvfs.h>
 # define df_stat(p, s)  (statvfs((p), (s)) == 0)
@@ -210,14 +210,17 @@ readline(QIOSTATE *qp)
 int
 main(int argc, char *argv[])
 {
-    int option, i, count;
+    int option, i;
     unsigned long total;
     QIOSTATE *qp;
     char *active, *group, *line, *p;
+    struct overview *overview = NULL;
+    struct overview_group stats;
+    float used;
     char *file = NULL;
     bool inode = false;
-    bool overview = false;
     bool ovcount = false;
+    bool ovused = false;
     bool use_filesystems = false;
 
     while ((option = getopt(argc, argv, "hinof:F")) != EOF) {
@@ -234,7 +237,7 @@ main(int argc, char *argv[])
             ovcount = true;
             break;
         case 'o':
-            overview = true;
+            ovused = true;
             break;
         case 'f':
             if (file != NULL)
@@ -254,7 +257,7 @@ main(int argc, char *argv[])
     argc -= optind;
     argv += optind;
 
-    if (argc == 0 && !overview && !ovcount && file == NULL)
+    if (argc == 0 && !ovused && !ovcount && file == NULL)
         die(usage);
 
     /* Set the program name now rather than earlier so that it doesn't get
@@ -265,7 +268,7 @@ main(int argc, char *argv[])
        one was given, just print out the number without the path or any
        explanatory text; this mode is used by e.g. innwatch.  Otherwise,
        format things nicely. */
-    if (argc == 1 && !overview && !ovcount && file == NULL) {
+    if (argc == 1 && !ovused && !ovcount && file == NULL) {
         printspace(argv[0], inode, false);
         printf("\n");
     } else {
@@ -290,18 +293,19 @@ main(int argc, char *argv[])
 
     /* If we're going to be getting information from overview, do the icky
        initialization stuff. */
-    if (overview || ovcount) {
+    if (ovused || ovcount) {
         if (!use_filesystems)
             if (!innconf_read(NULL))
                 exit(1);
-        if (!OVopen(OV_READ))
-            die("OVopen failed");
+        overview = overview_open(OV_READ);
+        if (overview == NULL)
+            die("can't open overview");
     }
 
     /* For the count, we have to troll through the active file and query the
        overview backend for each group. */
     if (ovcount) {
-        active = concatpath(innconf->pathdb, _PATH_ACTIVE);
+        active = concatpath(innconf->pathdb, INN_PATH_ACTIVE);
         qp = QIOopen(active);
         if (qp == NULL)
             sysdie("can't open %s", active);
@@ -312,8 +316,8 @@ main(int argc, char *argv[])
             p = strchr(group, ' ');
             if (p != NULL)
                 *p = '\0';
-            if (OVgroupstats(group, NULL, NULL, &count, NULL))
-		total += count;
+            if (overview_group(overview, group, &stats))
+                total += stats.count;
             group = QIOread(qp);
         }
         QIOclose(qp);
@@ -322,14 +326,15 @@ main(int argc, char *argv[])
 
     /* Percentage used is simpler, but only some overview methods understand
        that query. */
-    if (overview) {
-        if (OVctl(OVSPACE, &count)) {
-            if (count == -1)
-                printf("Space used is meaningless for the %s method\n",
-                       innconf->ovmethod);
-            else
-                printf("%d%% overview space used\n", count);
-        }
+    if (ovused) {
+        used = overview_free_space(overview);
+        if (used < 0)
+            printf("Space used is meaningless for the %s method\n",
+                   innconf->ovmethod);
+        else
+            printf("%.2f%% overview space used\n", used);
     }
+    if (overview != NULL)
+        overview_close(overview);
     exit(0);
 }
