@@ -1,4 +1,4 @@
-/*  $Id: expire.c 8574 2009-08-18 13:51:41Z iulius $
+/*  $Id: expire.c 9019 2010-03-19 21:27:15Z iulius $
 **
 **  Expire news articles.
 */
@@ -154,7 +154,7 @@ static bool EXPgetnum(int line, char *word, time_t *v, const char *name)
 		break;
 	    SawDot = true;
 	}
-	else if (!CTYPE(isdigit, (int)*p))
+	else if (!isdigit((unsigned char) *p))
 	    break;
     if (*p) {
         warn("bad '%c' character in %s field on line %d", *p, name, line);
@@ -202,7 +202,7 @@ static bool EXPreadfile(FILE *F)
 	else
 	    p = buff + strlen(buff);
 	while (--p >= buff) {
-	    if (isspace((int)*p))
+	    if (isspace((unsigned char) *p))
                 *p = '\0';
             else
                 break;
