@@ -1,4 +1,4 @@
-/*  $Id: expireover.c 6708 2004-05-16 19:59:26Z rra $
+/*  $Id: expireover.c 8570 2009-08-17 19:05:28Z iulius $
 **
 **  Expire the overview database.
 **
@@ -11,21 +11,21 @@
 #include "config.h"
 #include "clibrary.h"
 #include <errno.h>
-#include <pwd.h>
 #include <signal.h>
 #include <syslog.h>
 #include <time.h>
 
 #include "inn/innconf.h"
+#include "inn/libinn.h"
 #include "inn/messages.h"
+#include "inn/newsuser.h"
+#include "inn/ov.h"
+#include "inn/paths.h"
 #include "inn/qio.h"
-#include "libinn.h"
-#include "ov.h"
-#include "paths.h"
-#include "storage.h"
+#include "inn/storage.h"
 
 static const char usage[] = "\
-Usage: expireover [-ekNpqs] [-w offset] [-z rmfile] [-Z lowmarkfile]\n";
+Usage: expireover [-ekNpqs] [-f file] [-w offset] [-z rmfile] [-Z lowmarkfile]\n";
 
 /* Set to 1 if we've received a signal; expireover then terminates after
    finishing the newsgroup that it's working on (this prevents corruption of
@@ -38,7 +38,7 @@ static volatile sig_atomic_t signalled = 0;
 **  behavior after receiving a signal so that repeating the signal will kill
 **  the program immediately.
 */
-static RETSIGTYPE
+static void
 fatal_signal(int sig)
 {
     signalled = 1;
@@ -46,25 +46,6 @@ fatal_signal(int sig)
 }
 
 
-/*
-**  Change to the news user if possible, and if not, die.  Used for operations
-**  that may create new database files so as not to mess up the ownership.
-*/
-static void
-setuid_news(void)
-{
-    struct passwd *pwd;
-
-    pwd = getpwnam(NEWSUSER);
-    if (pwd == NULL)
-        die("can't resolve %s to a UID (account doesn't exist?)", NEWSUSER);
-    if (getuid() == 0)
-        setuid(pwd->pw_uid);
-    if (getuid() != pwd->pw_uid)
-        die("must be run as %s", NEWSUSER);
-}
-
-
 int
 main(int argc, char *argv[])
 {
@@ -142,8 +123,8 @@ main(int argc, char *argv[])
     if (!innconf_read(NULL))
         exit(1);
 
-    /* Change to the news user if necessary. */
-    setuid_news();
+    /* Change to the runasuser user and runasgroup group if necessary. */
+    ensure_news_user_grp(true, true);
 
     /* Initialize the lowmark file, if one was requested. */
     if (lowmark_path != NULL) {
@@ -157,7 +138,7 @@ main(int argc, char *argv[])
     /* Set up the path to the list of newsgroups we're going to use and open
        that file.  This could be stdin. */
     if (active_path == NULL) {
-        active_path = concatpath(innconf->pathdb, _PATH_ACTIVE);
+        active_path = concatpath(innconf->pathdb, INN_PATH_ACTIVE);
         purge_deleted = true;
     }
     if (strcmp(active_path, "-") == 0) {
@@ -172,7 +153,7 @@ main(int argc, char *argv[])
     free(active_path);
 
     /* open up the history manager */
-    path = concatpath(innconf->pathdb, _PATH_HISTORY);
+    path = concatpath(innconf->pathdb, INN_PATH_HISTORY);
     history = HISopen(path, innconf->hismethod, HIS_RDONLY);
     free(path);
 
