@@ -1,4 +1,4 @@
-/*  $Id: fastrm.c 9024 2010-03-21 16:49:30Z iulius $
+/*  $Id: fastrm.c 9844 2015-05-02 16:30:45Z iulius $
 **
 **  Delete a list of filenames or tokens from stdin.
 **
@@ -273,7 +273,6 @@ process_line(QIOSTATE *qp, int *queued, int *deleted)
                     dlen = strlen(line);
                     dir = xstrdup(line);
                 } else {
-                    p = line;
                     dlen = -1;
                     dir = NULL;
                 }
@@ -481,7 +480,7 @@ setup_dir(char *dir, int filecount)
 
        FIXME: It's too hard to figure out what this code does.  It needs to be
        rewritten. */
-    if (p != '\0' && relative_threshold > 0) {
+    if (*p != '\0' && relative_threshold > 0) {
         depth = slashcount(q);
         if (depth <= relative_threshold) {
             while (p > absolute && *--p != '/')
