@@ -1,4 +1,4 @@
-/*  $Id: inews.c 9521 2013-07-31 17:27:33Z iulius $
+/*  $Id: inews.c 9911 2015-07-04 21:32:56Z iulius $
 **
 **  Send an article (prepared by someone on the local site) to the
 **  master news server.
@@ -6,13 +6,17 @@
 
 #include "config.h"
 #include "clibrary.h"
-#include "portable/time.h"
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <pwd.h>
 #include <sys/stat.h>
 
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
+
 #include "inn/innconf.h"
 #include "inn/libinn.h"
 #include "inn/messages.h"
@@ -406,10 +410,7 @@ CheckControl(char *ctrl)
 	  || strcasecmp(ctrl, "ihave")       == 0
 	  || strcasecmp(ctrl, "sendme")      == 0
 	  || strcasecmp(ctrl, "newgroup")    == 0
-	  || strcasecmp(ctrl, "rmgroup")     == 0
-	  || strcasecmp(ctrl, "sendsys")     == 0
-	  || strcasecmp(ctrl, "senduuname")  == 0
-	  || strcasecmp(ctrl, "version")     == 0) {
+	  || strcasecmp(ctrl, "rmgroup")     == 0) {
 	if (!AnAdministrator())
             die("ask your news administrator to do the %s for you", ctrl);
     }
@@ -662,7 +663,6 @@ ProcessHeaders(bool AddOrg, int linecount, struct passwd *pwp)
 static char *
 AppendSignature(bool UseMalloc, char *article, char *homedir, int *linesp)
 {
-    static char	NOSIG[] = "Can't add your .signature (%s), article not posted";
     int		i;
     int		length;
     size_t      artsize;
@@ -678,7 +678,8 @@ AppendSignature(bool UseMalloc, char *article, char *homedir, int *linesp)
     if ((F = fopen(buff, "r")) == NULL) {
 	if (errno == ENOENT)
 	    return article;
-	fprintf(stderr, NOSIG, strerror(errno));
+        fprintf(stderr, "Can't add your .signature (%s), article not posted",
+                strerror(errno));
 	QuitServer(1);
     }
 
