@@ -1,4 +1,4 @@
-/*  $Id: rnews.c 9024 2010-03-21 16:49:30Z iulius $
+/*  $Id: rnews.c 9911 2015-07-04 21:32:56Z iulius $
 **
 **  A front-end for InterNetNews.
 **
@@ -9,14 +9,15 @@
 
 #include "config.h"
 #include "clibrary.h"
-#include "portable/wait.h"
 #include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <syslog.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 
+#include "inn/fdflag.h"
 #include "inn/innconf.h"
 #include "inn/libinn.h"
 #include "inn/messages.h"
@@ -169,10 +170,14 @@ Reject(const char *article, size_t length UNUSED, const char *reason,
     int fd;
 #endif	/* defined(DO_RNEWS_SAVE_BAD) */
 
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
     notice(reason, arg);
+#pragma GCC diagnostic warning "-Wformat-nonliteral"
     if (Verbose) {
 	fprintf(stderr, "%s: ", InputFile);
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
 	fprintf(stderr, reason, arg);
+#pragma GCC diagnostic warning "-Wformat-nonliteral"
 	fprintf(stderr, " [%.40s...]\n", article);
     }
 
@@ -434,12 +439,13 @@ ReadBytecount(int fd, int artsize)
        seems like the best of a set of bad options; Reject would save the
        article into bad and then someone might reprocess it, leaving us with
        accepting the truncated version. */
-    for (p = article, left = artsize; left; p += i, left -= i)
-	if ((i = read(fd, p, left)) <= 0) {
-	    i = errno;
+    for (p = article, left = artsize; left; p += i, left -= i) {
+        i = read(fd, p, left);
+        if (i <= 0) {
             warn("cannot read, wanted %d got %d", artsize, artsize - left);
-	    return true;
-	}
+            return true;
+        }
+    }
     if (p[-1] != '\n') {
 	*p++ = '\n';
         artsize++;
@@ -855,10 +861,7 @@ int main(int ac, char *av[])
        other setups where rnews might be setuid news or be run by other
        processes in the news group. */
     if (getuid() == 0 || geteuid() == 0) {
-        uid_t uid;
-
-        get_news_uid_gid(&uid, false, true);
-        setuid(uid);
+        ensure_news_user(true);
     }
 
     if (!innconf_read(NULL))
@@ -935,8 +938,8 @@ int main(int ac, char *av[])
 			CantConnect(buff, mode, fd);
 	}
     }
-    close_on_exec(fileno(FromServer), true);
-    close_on_exec(fileno(ToServer), true);
+    fdflag_close_exec(fileno(FromServer), true);
+    fdflag_close_exec(fileno(ToServer), true);
 
     /* Execute the command. */
     if (mode == 'U')
