@@ -1,4 +1,4 @@
-/*  $Id: sys2nf.c 6135 2003-01-19 01:15:40Z rra $
+/*  $Id: sys2nf.c 9019 2010-03-19 21:27:15Z iulius $
 **
 **  Read a C news "sys" file and split it up into a set of INN
 **  newsfeeds entries.  Also works with B news.
@@ -14,8 +14,9 @@
 #include <sys/stat.h>
 
 #include "inn/innconf.h"
-#include "libinn.h"
-#include "nntp.h"
+#include "inn/libinn.h"
+#include "inn/nntp.h"
+#include "inn/paths.h"
 
 #define TEMPFILE	":tmp"
 static char		**Groups;
@@ -86,7 +87,7 @@ ReadSys(sys)
 		break;
 	    }
 	    if (*p == '\\' && p[1] == '\n')
-		while (*++p && CTYPE(isspace, *p))
+		while (*++p && isspace((unsigned char) *p))
 		    continue;
 	    else
 		*to++ = *p++;
@@ -174,7 +175,7 @@ DoSub(F, p)
         "ba", "ca", "dc", "ne", "ny", "tx",
 
         "info", "mail", "opinions", "uunet"
-    }
+    };
 
     if ((s = strtok(p, ",")) == NULL)
 	return;
@@ -183,7 +184,7 @@ DoSub(F, p)
     len = 8 + 1 + 2;
     do {
         for (matched = false, i = 0; i < ARRAY_SIZE(distributions); i++)
-            if (strcmp(s, distributions[i]) == 0) {
+            if (strcasecmp(s, distributions[i]) == 0) {
                 matched = true;
                 break;
             }
@@ -209,9 +210,9 @@ DoSub(F, p)
 	    s++;
 	}
 
-	SawAll = (strcmp(s, "all") == 0);
+	SawAll = (strcasecmp(s, "all") == 0);
 	if (SawAll)
-	    s = SawBang ? "*" : "*,!control";
+	    s = SawBang ? "*" : "*,!control,!control.*";
 	len += strlen(s);
 	fprintf(F, "%s", s);
 
@@ -259,7 +260,7 @@ main(ac, av)
     if (!innconf_read(NULL))
         exit(1);
     /* Set defaults. */
-    act = "/usr/local/lib/newslib/active";
+    act = concatpath(innconf->pathdb, INN_PATH_ACTIVE);
     sys = "sys";
     dir = "feeds";
     while ((i = getopt(ac, av, "a:s:d:")) != EOF)
