@@ -1,4 +1,4 @@
-/*  $Id: innconf.h 9162 2011-01-23 10:24:23Z iulius $
+/*  $Id: innconf.h 9752 2014-12-01 20:22:57Z iulius $
 **
 **  inn.conf parser interface.
 **
@@ -99,10 +99,11 @@ struct innconf {
     unsigned long keymaxwords;  /* Max count of interesting words */
 
     /* Posting */
-    bool addnntppostingdate;    /* Add NNTP-Posting-Date: to posts */
-    bool addnntppostinghost;    /* Add NNTP-Posting-Host: to posts */
+    bool addinjectiondate;      /* Add Injection-Date: to posts */
+    bool addinjectionpostingaccount; /* Add posting-account attribute to Injection-Info: to posts */
+    bool addinjectionpostinghost; /* Add posting-host attribute to Injection-Info: to posts */
     bool checkincludedtext;     /* Reject if too much included text */
-    char *complaints;           /* Address for X-Complaints-To: */
+    char *complaints;           /* Address for mail-complaints-to attribute to Injection-Info: */
     char *fromhost;             /* Host for the From: line */
     unsigned long localmaxartsize; /* Max article size of local postings */
     char *moderatormailer;      /* Default host to mail moderated articles */
@@ -122,12 +123,19 @@ struct innconf {
     unsigned long backofftrigger;  /* Number of postings before triggered */
 
     /* Reading and posting -- SSL and TLS support */
-#ifdef HAVE_SSL
+    /* Do not test HAVE_OPENSSL.  This relieves customers of /usr/include/inn
+     * from the need to guess whether INN was built with SSL/TLS support in
+     * order to get a header that matches the installed libraries.
+     */
     char *tlscafile;            /* Path to a certificate authority file */
     char *tlscapath;            /* Path to a directory of CA certificates */
     char *tlscertfile;          /* Path to the SSL certificate to use */
     char *tlskeyfile;           /* Path to the key for the certificate */
-#endif /* HAVE_SSL */
+    char *tlsciphers;           /* OpenSSL-style cipher string */
+    bool tlscompression;        /* Turn TLS compression on/off */
+    char *tlseccurve;           /* ECDH curve name */
+    bool tlspreferserverciphers; /* Make server select the cipher */
+    struct vector *tlsprotocols; /* List of supported TLS versions */
 
     /* Monitoring */
     bool doinnwatch;            /* Start innwatch from rc.news? */
