@@ -1,10 +1,11 @@
-/*  $Id: art.c 9378 2011-09-13 18:47:23Z iulius $
+/*  $Id: art.c 9896 2015-06-14 10:09:47Z iulius $
 **
 **  Article-processing.
 */
 
 #include "config.h"
 #include "clibrary.h"
+#include "portable/macros.h"
 #include <sys/uio.h>
 
 #include "inn/innconf.h"
@@ -130,7 +131,7 @@ bool
 ARTreadschema(void)
 {
   const struct cvector *standardoverview;
-  const struct vector  *extraoverview;
+  struct vector        *extraoverview;
   unsigned int         i;
   ARTOVERFIELD         *fp;
   const ARTHEADER      *hp;
@@ -182,6 +183,7 @@ ARTreadschema(void)
   }
 
   fp->Header = NULL;
+  vector_free(extraoverview);
 
   return ok;
 }
@@ -301,20 +303,22 @@ ARTlog(const ARTDATA *data, char code, const char *text)
   const HDRCONTENT *hc = data->HdrContent;
   int i;
   bool Done;
+  time_t t;
 
   TMRstart(TMR_ARTLOG);
   /* We could be a bit faster by not dividing Now.usec by 1000,
    * but who really wants to log at the Microsec level? */
   Done = code == ART_ACCEPT || code == ART_JUNK;
+  t = Now.tv_sec;
   if (text)
     i = fprintf(Log, "%.15s.%03d %c %s %s %s%s",
-      ctime(&Now.tv_sec) + 4, (int)(Now.tv_usec / 1000), code,
+      ctime(&t) + 4, (int)(Now.tv_usec / 1000), code,
       data->Feedsite != NULL ? data->Feedsite : "(null)",
       HDR_FOUND(HDR__MESSAGE_ID) ? HDR(HDR__MESSAGE_ID) : "(null)",
       text, Done ? "" : "\n");
   else
     i = fprintf(Log, "%.15s.%03d %c %s %s%s",
-      ctime(&Now.tv_sec) + 4, (int)(Now.tv_usec / 1000), code,
+      ctime(&t) + 4, (int)(Now.tv_usec / 1000), code,
       data->Feedsite != NULL ? data->Feedsite : "(null)",
       HDR_FOUND(HDR__MESSAGE_ID) ? HDR(HDR__MESSAGE_ID) : "(null)",
       Done ? "" : "\n");
@@ -442,7 +446,7 @@ ARTstore(CHANNEL *cp)
   ARTDATA	*data = &cp->Data;
   HDRCONTENT	*hc = data->HdrContent;
   const char	*p;
-  ARTHANDLE	arth;
+  ARTHANDLE	arth = ARTHANDLE_INITIALIZER;
   int		i, j, iovcnt = 0;
   long		headersize = 0;
   TOKEN		result;
@@ -649,7 +653,7 @@ ARTcheckheader(CHANNEL *cp, int size)
   }
 
   /* See if this is a system header.  A fairly tightly-coded binary search. */
-  c = islower((unsigned char) *header) ? toupper(*header) : *header;
+  c = islower((unsigned char) *header) ? toupper((unsigned char) *header) : *header;
   for (*colon = '\0', tp = ARTheadertree; tp; ) {
     if ((i = c - tp->Name[0]) == 0 && (i = strcasecmp(header, tp->Name)) == 0)
       break;
@@ -876,7 +880,7 @@ ARTparseheader(CHANNEL *cp)
                 data->Body = i + 2;
                 cp->State = CSgetbody;
                 ARTparsebody(cp);
-		return;
+                return;
             }
         } else {
             data->CRwithoutLF++;
@@ -1688,7 +1692,7 @@ ARTpropagate(ARTDATA *data, const char **hops, int hopcount, char **list,
       } else if (HDR_FOUND(HDR__INJECTION_INFO)) {
         begin = (char *) skip_cfws(HDR(HDR__INJECTION_INFO));
 
-        if (begin == '\0')
+        if (*begin == '\0')
           continue;
 
         /* The path identity ends with ';' or CFWS. */
@@ -1999,7 +2003,7 @@ ARTpost(CHANNEL *cp)
   }
   hopcount = ARTparsepath(HDR(HDR__PATH), HDR_LEN(HDR__PATH), &data->Path);
   if (hopcount == 0) {
-    snprintf(cp->Error, sizeof(cp->Error), "%d illegal path element",
+    snprintf(cp->Error, sizeof(cp->Error), "%d Illegal path element",
              ihave ? NNTP_FAIL_IHAVE_REJECT : NNTP_FAIL_TAKETHIS_REJECT);
     /* We do not remember the message-ID of this article because another
      * peer may send it with a good Path: header. */
@@ -2139,7 +2143,7 @@ ARTpost(CHANNEL *cp)
   /* If we limit what distributions we get, see if we want this one. */
   if (HDR_FOUND(HDR__DISTRIBUTION)) {
     if (HDR(HDR__DISTRIBUTION)[0] == ',') {
-      snprintf(cp->Error, sizeof(cp->Error), "%d bogus distribution \"%s\"",
+      snprintf(cp->Error, sizeof(cp->Error), "%d Bogus distribution \"%s\"",
                ihave ? NNTP_FAIL_IHAVE_REJECT : NNTP_FAIL_TAKETHIS_REJECT,
                MaxLength(HDR(HDR__DISTRIBUTION), HDR(HDR__DISTRIBUTION)));
       ARTlog(data, ART_REJECT, cp->Error);
@@ -2523,7 +2527,7 @@ ARTpost(CHANNEL *cp)
   }
   if (token.type == TOKEN_EMPTY) {
     syslog(L_ERROR, "%s cant store article: %s", LogName, SMerrorstr);
-    snprintf(cp->Error, sizeof(cp->Error), "%d cant store article",
+    snprintf(cp->Error, sizeof(cp->Error), "%d Can't store article",
              ihave ? NNTP_FAIL_IHAVE_DEFER : NNTP_FAIL_ACTION);
     /* Do not remember the message-ID of the article because we want
      * it to be received again later. */
@@ -2564,7 +2568,7 @@ ARTpost(CHANNEL *cp)
     i = errno;
     syslog(L_ERROR, "%s cant write history %s %m", LogName,
       HDR(HDR__MESSAGE_ID));
-    snprintf(cp->Error, sizeof(cp->Error), "%d cant write history, %s",
+    snprintf(cp->Error, sizeof(cp->Error), "%d Can't write history, %s",
              ihave ? NNTP_FAIL_IHAVE_DEFER : NNTP_FAIL_ACTION,
              strerror(errno));
     ARTlog(data, ART_REJECT, cp->Error);
