@@ -1,4 +1,4 @@
-/*  $Id: chan.c 9470 2013-05-20 16:34:54Z iulius $
+/*  $Id: chan.c 9905 2015-06-20 20:51:22Z iulius $
 **
 **  I/O channel (and buffer) processing.
 **
@@ -22,6 +22,7 @@
 # include <sys/select.h>
 #endif
 
+#include "inn/fdflag.h"
 #include "inn/innconf.h"
 #include "inn/network.h"
 #include "innd.h"
@@ -203,14 +204,14 @@ CHANcreate(int fd, enum channel_type type, enum channel_state state,
     HashClear(&cp->CurrentMessageIDHash);
     ARTprepare(cp);
 
-    close_on_exec(fd, true);
+    fdflag_close_exec(fd, true);
 
 #ifndef _HPUX_SOURCE
     /* Stupid HPUX 11.00 has a broken listen/accept where setting the listen
        socket to nonblocking prevents you from successfully setting the
        socket returned by accept(2) back to blocking mode, no matter what,
        resulting in all kinds of funny behaviour, data loss, etc. etc.  */
-    if (nonblocking(fd, true) < 0 && errno != ENOTSOCK && errno != ENOTTY)
+    if (!fdflag_nonblocking(fd, true) && errno != ENOTSOCK && errno != ENOTTY)
         syswarn("%s cant nonblock %d", LogName, fd);
 #endif
 
@@ -539,7 +540,7 @@ CHANNEL *
 CHANiter(int *ip, enum channel_type type)
 {
     CHANNEL *cp;
-    int i = *ip;
+    int i;
 
     for (i = *ip; i >= 0 && i < channels.table_size; i++) {
         cp = &channels.table[i];
@@ -570,6 +571,21 @@ CHANresetlast(int fd)
 }
 
 
+/*
+**  When removing a channel from the sleep mask, we want to lower the last
+**  file descriptor if we removed the highest one.  Called from SCHANremove.
+*/
+static void
+CHANresetlastsleeping(int fd)
+{
+    if (fd == channels.max_sleep_fd) {
+        while (   !FD_ISSET(channels.max_sleep_fd, &channels.sleep_set)
+               && channels.max_sleep_fd > 1)
+            channels.max_sleep_fd--;
+    }
+}
+
+
 /*
 **  Mark a channel as an active reader.
 */
@@ -629,8 +645,6 @@ SCHANadd(CHANNEL *cp, time_t wake, void *event, innd_callback_func waker,
 void
 SCHANremove(CHANNEL *cp)
 {
-    int fd;
-
     if (!CHANsleeping(cp))
         return;
     FD_CLR(cp->fd, &channels.sleep_set);
@@ -638,12 +652,7 @@ SCHANremove(CHANNEL *cp)
     cp->Waketime = 0;
 
     /* If this was the highest descriptor, get a new highest. */
-    if (cp->fd == channels.max_sleep_fd) {
-        fd = channels.max_sleep_fd;
-        while (!FD_ISSET(fd, &channels.sleep_set) && fd > 1)
-            fd--;
-        channels.max_sleep_fd = fd;
-    }
+    CHANresetlastsleeping(cp->fd);
 }
 
 
@@ -1164,8 +1173,13 @@ CHANreadloop(void)
                 tv.tv_sec = innconf->timer;
             }
         }
+
+        /* Mask signals when not in select to prevent a signal handler
+           from accessing data that the main code is mutating. */
         TMRstart(TMR_IDLE);
+        xsignal_unmask();
         count = select(channels.max_fd + 1, &rdfds, &wrfds, NULL, &tv);
+        xsignal_mask();
         TMRstop(TMR_IDLE);
 
         if (count < 0) {
@@ -1276,12 +1290,21 @@ CHANreadloop(void)
                 if (cp->Type == CTfree) {
                     warn("%s %d free but was in SMASK", CHANname(cp), fd);
                     FD_CLR(fd, &channels.sleep_set);
+                    channels.sleep_count--;
+                    CHANresetlastsleeping(fd);
                     close(fd);
                     cp->fd = -1;
                 } else {
                     cp->LastActive = Now.tv_sec;
                     SCHANremove(cp);
-                    (*cp->Waker)(cp);
+                    if (cp->Waker != NULL) {
+                        (*cp->Waker)(cp);
+                    } else {
+                        name = CHANname(cp);
+                        warn("%s %d sleeping without Waker", name, fd);
+                        SITEchanclose(cp);
+                        CHANclose(cp, name);
+                    }
                 }
             }
 
