@@ -1,4 +1,4 @@
-/*  $Id: rc.c 9625 2014-03-16 13:24:22Z iulius $
+/*  $Id: rc.c 9897 2015-06-14 10:10:37Z iulius $
 **
 **  Routines for the remote connect channel.  Create an Internet stream
 **  socket that processes connect to.  If the incoming site is not one of
@@ -11,8 +11,10 @@
 #include <errno.h>
 #include <netdb.h>
 
+#include "inn/fdflag.h"
 #include "inn/innconf.h"
 #include "inn/network.h"
+#include "inn/network-innbind.h"
 #include "inn/vector.h"
 #include "innd.h"
 
@@ -64,7 +66,7 @@ static char		*RCslaveflag;
 static char		*RCnnrpd = NULL;
 static char		*RCnntpd = NULL;
 static CHANNEL		**RCchan;
-static int		chanlimit;
+static unsigned int	chanlimit;
 static REMOTEHOST_DATA	*RCpeerlistfile;
 static REMOTEHOST	*RCpeerlist;
 static int		RCnpeerlist;
@@ -149,19 +151,19 @@ GoodIdent(int fd, char *identd)
 #ifdef HAVE_INET6
     if (s_local->sa_family == AF_INET6)
     {
-	port1 = ntohs(((struct sockaddr_in6 *) s_local)->sin6_port);
-	port2 = ntohs(((struct sockaddr_in6 *) s_distant)->sin6_port);
-	((struct sockaddr_in6 *) s_local)->sin6_port = 0;
-	((struct sockaddr_in6 *) s_distant)->sin6_port = htons(PORT_IDENTD);
+        port1 = ntohs(((struct sockaddr_in6 *) (void *) s_local)->sin6_port);
+        port2 = ntohs(((struct sockaddr_in6 *) (void *) s_distant)->sin6_port);
+        ((struct sockaddr_in6 *) (void *) s_local)->sin6_port = 0;
+        ((struct sockaddr_in6 *) (void *) s_distant)->sin6_port = htons(PORT_IDENTD);
 	ident_fd = socket(PF_INET6, SOCK_STREAM, 0);
     } else
 #endif
     if (s_local->sa_family == AF_INET)
     {
-	port1 = ntohs(((struct sockaddr_in *) s_local)->sin_port);
-        port2 = ntohs(((struct sockaddr_in *) s_distant)->sin_port);
-	((struct sockaddr_in *) s_local)->sin_port = 0;
-	((struct sockaddr_in *) s_distant)->sin_port = htons(PORT_IDENTD);
+        port1 = ntohs(((struct sockaddr_in *) (void *) s_local)->sin_port);
+        port2 = ntohs(((struct sockaddr_in *) (void *) s_distant)->sin_port);
+        ((struct sockaddr_in *) (void *) s_local)->sin_port = 0;
+        ((struct sockaddr_in *) (void *) s_distant)->sin_port = htons(PORT_IDENTD);
 	ident_fd = socket(PF_INET, SOCK_STREAM, 0);
     } else
     {
@@ -381,7 +383,7 @@ RChandoff(int fd, HANDOFF h)
         syslog(L_ERROR, "fd %d cant setsockopt(KEEPALIVE) %m", fd);
 #endif /* defined(SOL_SOCKET) && defined(SO_KEEPALIVE) */
 
-    if (nonblocking(fd, false) < 0)
+    if (!fdflag_nonblocking(fd, false))
 	syslog(L_ERROR, "%s cant nonblock %d in RChandoff %m", LogName, fd);
     switch (h) {
     default:
@@ -425,6 +427,7 @@ RCreader(CHANNEL *cp)
     struct sockaddr_storage	remote;
     socklen_t		size;
     int                 i;
+    unsigned int        j;
     REMOTEHOST          *rp;
     CHANNEL		*new;
     char		*name;
@@ -437,12 +440,12 @@ RCreader(CHANNEL *cp)
     char		buff[SMBUF];
     char                addr[INET6_ADDRSTRLEN];
 
-    for (i = 0 ; i < chanlimit ; i++) {
-	if (RCchan[i] == cp) {
+    for (j = 0 ; j < chanlimit ; j++) {
+	if (RCchan[j] == cp) {
 	    break;
 	}
     }
-    if (i == chanlimit) {
+    if (j == chanlimit) {
 	syslog(L_ERROR, "%s internal RCreader wrong channel 0x%p",
 		LogName, (void *)cp);
 	return;
@@ -455,11 +458,6 @@ RCreader(CHANNEL *cp)
 	    syslog(L_ERROR, "%s cant accept RCreader %m", LogName);
 	return;
     }
-    if (!network_kill_options(fd, (struct sockaddr *) &remote)) {
-        if (close(fd) < 0)
-            syslog(L_ERROR, "%s cant close %d %m", LogName, fd);
-        return;
-    }
 
     /* If RemoteTimer is not zero, then check the limits on incoming
        connections on a total and per host basis.
@@ -661,7 +659,6 @@ RCreaddata(int *num, FILE *F, bool *toolong)
   char	*s;
   char	*t;
   char	*word;
-  bool	flag;
 
   *toolong = false;
   if (*RCbuff == '\0') {
@@ -673,7 +670,6 @@ RCreaddata(int *num, FILE *F, bool *toolong)
       return (NULL); /* Line too long */
     }
   }
-  p = RCbuff;
   do {
      /* Ignore blank and comment lines. */
      if ((p = strchr(RCbuff, '\n')) != NULL)
@@ -683,19 +679,17 @@ RCreaddata(int *num, FILE *F, bool *toolong)
 	   *p = '\0';
      }
      for (p = RCbuff; *p == ' ' || *p == '\t' ; p++);
-     flag = true;
      if (*p == '\0' && !feof (F)) {
-       flag = false;
        fgets(RCbuff, sizeof RCbuff, F);
        (*num)++;
        if (strlen (RCbuff) == sizeof RCbuff) {
-	 *toolong = true;
-	 return (NULL); /* Line too long */
+           *toolong = true;
+           return (NULL); /* Line too long */
        }
        continue;
      }
      break;
-  } while (!feof (F) || !flag);
+  } while (!feof (F));
 
   if (*p == '"') { /* double quoted string ? */
     p++;
@@ -1519,7 +1513,7 @@ RCwritelist(char *filename)
         if (*p == '/')
 	   q = p + 1;
 
-    fprintf (F, "##  $Id: rc.c 9625 2014-03-16 13:24:22Z iulius $\n");
+    fprintf (F, "##  $Id: rc.c 9897 2015-06-14 10:10:37Z iulius $\n");
     fprintf (F, "##\n");
     fprintf (F, "##  %s -- Configuration of incoming news feeds\n", q);
     free(r);
@@ -1736,7 +1730,7 @@ void
 RCsetup(void)
 {
     CHANNEL *rc;
-    int count, i, start;
+    unsigned int count, i, start;
     int *fds;
     bool okay;
 
@@ -1745,19 +1739,23 @@ RCsetup(void)
        descriptors.  Two is probably always sufficient, but we can't tell for
        sure how many will be returned by getaddrinfo and this only happens on
        startup so doing a bit of memory allocation won't hurt. */
-    if (innconf->bindaddress == NULL && innconf->bindaddress6 == NULL)
-        network_bind_all(innconf->port, &fds, &count);
-    else {
+    if (innconf->bindaddress == NULL && innconf->bindaddress6 == NULL) {
+        network_innbind_all(SOCK_STREAM, innconf->port, &fds, &count);
+    } else {
         if (innconf->bindaddress != NULL && innconf->bindaddress6 != NULL)
             count = 2;
         else
             count = 1;
         fds = xmalloc(count * sizeof(int));
         i = 0;
-        if (innconf->bindaddress6 != NULL)
-            fds[i++] = network_bind_ipv6(innconf->bindaddress6, innconf->port);
-        if (innconf->bindaddress != NULL)
-            fds[i] = network_bind_ipv4(innconf->bindaddress, innconf->port);
+        if (innconf->bindaddress6 != NULL) {
+            fds[i++] = network_innbind_ipv6(SOCK_STREAM, innconf->bindaddress6,
+                                            innconf->port);
+        }
+        if (innconf->bindaddress != NULL) {
+            fds[i] = network_innbind_ipv4(SOCK_STREAM, innconf->bindaddress,
+                                          innconf->port);
+        }
     }
 
     /* Make sure that the RCchan array is of the appropriate size. */
@@ -1801,6 +1799,8 @@ RCsetup(void)
 
     /* Get the list of hosts we handle. */
     RCreadlist();
+
+    free(fds);
 }
 
 
@@ -1810,12 +1810,13 @@ RCsetup(void)
 void
 RCclose(void)
 {
-    REMOTEHOST	*rp;
-    int		i;
+    REMOTEHOST   *rp;
+    int          i;
+    unsigned int j;
 
-    for (i = 0 ; i < chanlimit ; i++) {
-	if (RCchan[i] != NULL) {
-	    CHANclose(RCchan[i], CHANname(RCchan[i]));
+    for (j = 0 ; j < chanlimit ; j++) {
+	if (RCchan[j] != NULL) {
+	    CHANclose(RCchan[j], CHANname(RCchan[j]));
 	} else {
 	    break;
 	}
