@@ -1,4 +1,4 @@
-/*  $Id: site.c,v 1.49.2.2 2004/05/16 20:26:54 rra Exp $
+/*  $Id: site.c 9911 2015-07-04 21:32:56Z iulius $
 **
 **  Routines to implement site-feeding.  Mainly working with channels to
 **  do buffering and determine what to send.
@@ -7,6 +7,7 @@
 #include "config.h"
 #include "clibrary.h"
 
+#include "inn/fdflag.h"
 #include "inn/innconf.h"
 #include "innd.h"
 
@@ -14,7 +15,7 @@
 static int	SITEcount;
 static int	SITEhead = NOSITE;
 static int	SITEtail = NOSITE;
-static char	SITEshell[] = _PATH_SH;
+static char	SITEshell[] = "/bin/sh";
 
 
 /*
@@ -42,32 +43,36 @@ SITEwritedone(CHANNEL *cp UNUSED)
 static bool
 SITEspool(SITE *sp, CHANNEL *cp)
 {
-    int			i;
-    char		buff[SPOOLNAMEBUFF];
-    char		*name;
+    int i;
+    char *name;
+    char *togo = NULL;
 
     name = sp->SpoolName;
     i = open(name, O_APPEND | O_CREAT | O_WRONLY, BATCHFILE_MODE);
     if (i < 0 && errno == EISDIR) {
-	FileGlue(buff, sp->SpoolName, '/', "togo");
-	name = buff;
-	i = open(buff, O_APPEND | O_CREAT | O_WRONLY, BATCHFILE_MODE);
+        togo = concatpath(sp->SpoolName, "togo");
+        name = togo;
+	i = open(name, O_APPEND | O_CREAT | O_WRONLY, BATCHFILE_MODE);
     }
     if (i < 0) {
 	i = errno;
 	syslog(L_ERROR, "%s cant open %s %m", sp->Name, name);
 	IOError("site batch file", i);
 	sp->Channel = NULL;
+        if (togo != NULL)
+            free(togo);
 	return false;
     }
-    if (cp) {
-      if (cp->fd >= 0)
-        /* syslog(L_ERROR, "DEBUG ERROR SITEspool trashed:%d %s:%d", cp->fd, sp->Name, i);
-	   CPU-eating bug, killed by kre. */
-	WCHANremove(cp);
-	RCHANremove(cp);
-	SCHANremove(cp);
-	close(cp->fd);
+    if (togo != NULL)
+        free(togo);
+    if (cp != NULL) {
+        /* Don't log a message here; it spewed into syslog. */
+        if (cp->fd >= 0) {
+            WCHANremove(cp);
+            RCHANremove(cp);
+            SCHANremove(cp);
+            close(cp->fd);
+        }
 	cp->fd = i;
 	return true;
     }
@@ -175,25 +180,29 @@ SITEbufferoldest(void)
 static bool
 SITECHANbilge(SITE *sp)
 {
-    int             fd;
-    int             i;
-    char            buff[SPOOLNAMEBUFF];
-    char           *name;
+    int fd;
+    int i;
+    char *name;
+    char *togo = NULL;
 
     name = sp->SpoolName;
     fd = open(name, O_APPEND | O_CREAT | O_WRONLY, BATCHFILE_MODE);
     if (fd < 0 && errno == EISDIR) {
-        FileGlue(buff, sp->SpoolName, '/', "togo");
-        name = buff;
-        fd = open(buff, O_APPEND | O_CREAT | O_WRONLY, BATCHFILE_MODE);
+        togo = concatpath(sp->SpoolName, "togo");
+        name = togo;
+        fd = open(togo, O_APPEND | O_CREAT | O_WRONLY, BATCHFILE_MODE);
     }
     if (fd < 0) {
 	int oerrno = errno ;
         syslog(L_ERROR, "%s cant open %s %m", sp->Name, name);
         IOError("site batch file",oerrno);
         sp->Channel = NULL;
+        if (togo != NULL)
+            free(togo);
         return false;
     }
+    if (togo != NULL)
+        free(togo);
     while (sp->Channel->Out.left > 0) {
         i = write(fd, &sp->Channel->Out.data[sp->Channel->Out.used],
                   sp->Channel->Out.left);
@@ -277,7 +286,7 @@ SITEflushcheck(SITE *sp, struct buffer *bp)
 	    WCHANadd(cp);
     }
 
-    cp->LastActive = Now.time;
+    cp->LastActive = Now.tv_sec;
 
     /* If we're a channel that's getting big, see if we need to spool. */
     if (sp->Type == FTfile || sp->StartSpooling == 0 || i < sp->StartSpooling)
@@ -305,7 +314,7 @@ SITEwrite(SITE *sp, const char *text)
     else {
 	if (sp->Channel == NULL)
 	    return;
-	sp->Channel->LastActive = Now.time;
+	sp->Channel->LastActive = Now.tv_sec;
 	bp = &sp->Channel->Out;
     }
     buffer_append(bp, PREFIX, strlen(PREFIX));
@@ -390,11 +399,18 @@ SITEwritefromflags(SITE *sp, ARTDATA *Data)
 	case FEED_PATH:
 	    if (Dirty)
 		buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
-	    if (!Data->Hassamepath)
-		buffer_append(bp, Path.data, Path.used);
-	    if (Data->AddAlias)
-		buffer_append(bp, Pathalias.data, Pathalias.used);
-	    buffer_append(bp, HDR(HDR__PATH), HDR_LEN(HDR__PATH));
+	    if (!Data->Hassamepath || Data->AddAlias || Pathcluster.used) {
+                if (Pathcluster.used)
+                    buffer_append(bp, Pathcluster.data, Pathcluster.used);
+                buffer_append(bp, Path.data, Path.used);
+                if (Data->AddAlias)
+                    buffer_append(bp, Pathalias.data, Pathalias.used);
+            }
+            if (Data->Hassamecluster)
+                buffer_append(bp, HDR(HDR__PATH) + Pathcluster.used,
+                    HDR_LEN(HDR__PATH) - Pathcluster.used);
+            else
+                buffer_append(bp, HDR(HDR__PATH), HDR_LEN(HDR__PATH));
 	    break;
 	case FEED_REPLIC:
 	    if (Dirty)
@@ -514,11 +530,15 @@ SITEsend(SITE *sp, ARTDATA *Data)
 	    strlcat(temp, sp->FNLnames.data, i + 1);
 	    strlcat(temp, &p[1], i + 1);
 	    *p = '*';
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
 	    snprintf(buff, sizeof(buff), temp, Data->TokenText);
+#pragma GCC diagnostic warning "-Wformat-nonliteral"
 	    free(temp);
-	}
-	else
+	} else {
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
 	    snprintf(buff, sizeof(buff), sp->Param, Data->TokenText);
+#pragma GCC diagnostic warning "-Wformat-nonliteral"
+        }
 
 	if (NeedShell(buff, (const char **)argv, (const char **)ARRAY_END(argv))) {
 	    argv[0] = SITEshell;
@@ -528,7 +548,7 @@ SITEsend(SITE *sp, ARTDATA *Data)
 	}
 
 	/* Start the process. */
-	i = Spawn(sp->Nice, 0, (int)fileno(Errlog), (int)fileno(Errlog), argv);
+	i = Spawn(sp->Nice, 0, fileno(Errlog), fileno(Errlog), argv);
 	if (i >= 0)
 	    PROCwatch(i, -1);
 	break;
@@ -586,7 +606,7 @@ SITEstartprocess(SITE *sp)
 	return false;
     }
 #endif
-    close_on_exec(pan[PIPE_WRITE], true);
+    fdflag_close_exec(pan[PIPE_WRITE], true);
 
     /* Set up the argument vector. */
     process = xstrdup(sp->Param);
@@ -624,7 +644,7 @@ SITEstartprocess(SITE *sp)
     syslog(L_ERROR, "%s cant spawn spooling %m", sp->Name);
     ip = xmalloc(sizeof(int));
     *ip = sp - Sites;
-    SCHANadd(sp->Channel, Now.time + innconf->chanretrytime, NULL,
+    SCHANadd(sp->Channel, Now.tv_sec + innconf->chanretrytime, NULL,
              SITEspoolwake, ip);
     return true;
 }
@@ -772,7 +792,7 @@ SITEchanclose(CHANNEL *cp)
 	    WCHANadd(sp->Channel);
 	    ip = xmalloc(sizeof(int));
 	    *ip = sp - Sites;
-	    SCHANadd(sp->Channel, Now.time + innconf->chanretrytime, NULL,
+	    SCHANadd(sp->Channel, Now.tv_sec + innconf->chanretrytime, NULL,
                      SITEspoolwake, ip);
 	    break;
 	}
@@ -997,6 +1017,7 @@ void
 SITEfree(SITE *sp)
 {
     SITE                *s;
+    HASHFEEDLIST        *hf, *hn;
     int                 new;
     int                 i;
     
@@ -1051,6 +1072,13 @@ SITEfree(SITE *sp)
 	sp->FNLnames.data = NULL;
 	sp->FNLnames.size = 0;
     }
+    if (sp->HashFeedList) {
+        for (hf = sp->HashFeedList; hf; hf = hn) {
+            hn = hf->next;
+            free(hf);
+        }
+        sp->HashFeedList = NULL;
+    }
 
     /* If this site was a master, find a new one. */
     if (sp->IsMaster) {
@@ -1114,118 +1142,98 @@ void
 SITEinfo(struct buffer *bp, SITE *sp, const bool Verbose)
 {
     static char		FREESITE[] = "<<No name>>\n\n";
-    char	        *p;
     CHANNEL	        *cp;
     const char		*sep;
-    char		buff[BUFSIZ];
 
     if (sp->Name == NULL) {
 	buffer_append(bp, FREESITE, strlen(FREESITE));
 	return;
     }
 
-    p = buff;
-    snprintf(buff, sizeof(buff), "%s%s:\t", sp->Name,
-             sp->IsMaster ? "(*)" : "");
-    p += strlen(p);
+    buffer_append_sprintf(bp, "%s%s:\t", sp->Name, sp->IsMaster ? "(*)" : "");
 
     if (sp->Type == FTfunnel) {
 	sp = &Sites[sp->Funnel];
-	sprintf(p, "funnel -> %s: ", sp->Name);
-	p += strlen(p);
+        buffer_append_sprintf(bp, "funnel -> %s: ", sp->Name);
     }
 
     switch (sp->Type) {
     default:
-	sprintf(p, "unknown feed type %d", sp->Type);
-	p += strlen(p);
+        buffer_append_sprintf(bp, "unknown feed type %d", sp->Type);
 	break;
     case FTerror:
     case FTfile:
-	p += strlen(strcpy(p, "file"));
-	if (sp->Buffered) {
-	    sprintf(p, " buffered(%lu)", (unsigned long) sp->Buffer.left);
-	    p += strlen(p);
-	}
+        buffer_append_sprintf(bp, "file");
+	if (sp->Buffered)
+            buffer_append_sprintf(bp, " buffered(%lu)",
+                           (unsigned long) sp->Buffer.left);
 	else if ((cp = sp->Channel) == NULL)
-	    p += strlen(strcpy(p, " no channel?"));
-	else {
-	    sprintf(p, " open fd=%d, in mem %lu", cp->fd,
-                    (unsigned long) cp->Out.left);
-	    p += strlen(p);
-	}
+            buffer_append_sprintf(bp, " no channel?");
+	else
+            buffer_append_sprintf(bp, " open fd=%d, in mem %lu", cp->fd,
+                           (unsigned long) cp->Out.left);
 	break;
     case FTchannel:
-	p += strlen(strcpy(p, "channel"));
-	goto Common;
+        buffer_append_sprintf(bp, "channel");
+	goto common;
     case FTexploder:
-	p += strlen(strcpy(p, "exploder"));
-Common:
-	if (sp->Process >= 0) {
-	    sprintf(p, " pid=%ld", (long) sp->pid);
-	    p += strlen(p);
-	}
+        buffer_append_sprintf(bp, "exploder");
+common:
+	if (sp->Process >= 0)
+            buffer_append_sprintf(bp, " pid=%ld", (long) sp->pid);
 	if (sp->Spooling)
-	    p += strlen(strcpy(p, " spooling"));
-	if ((cp = sp->Channel) == NULL)
-	    p += strlen(strcpy(p, " no channel?"));
-	else {
-	    sprintf(p, " fd=%d, in mem %lu", cp->fd,
-                    (unsigned long) cp->Out.left);
-	    p += strlen(p);
-	}
+            buffer_append_sprintf(bp, " spooling");
+        cp = sp->Channel;
+	if (cp == NULL)
+            buffer_append_sprintf(bp, " no channel?");
+	else
+            buffer_append_sprintf(bp, " fd=%d, in mem %lu", cp->fd,
+                           (unsigned long) cp->Out.left);
 	break;
     case FTfunnel:
-	p += strlen(strcpy(p, "recursive funnel"));
+        buffer_append_sprintf(bp, "recursive funnel");
 	break;
     case FTlogonly:
-	p += strlen(strcpy(p, "log only"));
+        buffer_append_sprintf(bp, "log only");
 	break;
     case FTprogram:
-	p += strlen(strcpy(p, "program"));
+        buffer_append_sprintf(bp, "program");
 	if (sp->FNLwantsnames)
-	    p += strlen(strcpy(p, " with names"));
+            buffer_append_sprintf(bp, " with names");
 	break;
     }
-    *p++ = '\n';
+    buffer_append(bp, "\n", 1);
     if (Verbose) {
 	sep = "\t";
 	if (sp->Buffered && sp->Flushpoint) {
-	    sprintf(p, "%sFlush @ %ld", sep, sp->Flushpoint);
-	    p += strlen(p);
+            buffer_append_sprintf(bp, "%sFlush @ %lu", sep,
+                           (unsigned long) sp->Flushpoint);
 	    sep = "; ";
 	}
 	if (sp->StartWriting || sp->StopWriting) {
-	    sprintf(p, "%sWrite [%ld..%ld]", sep,
-		sp->StopWriting, sp->StartWriting);
-	    p += strlen(p);
+            buffer_append_sprintf(bp, "%sWrite [%ld..%ld]", sep,
+                           sp->StopWriting, sp->StartWriting);
 	    sep = "; ";
 	}
 	if (sp->StartSpooling) {
-	    sprintf(p, "%sSpool @ %ld", sep, sp->StartSpooling);
-	    p += strlen(p);
+            buffer_append_sprintf(bp, "%sSpool @ %ld", sep, sp->StartSpooling);
 	    sep = "; ";
 	}
 	if (sep[0] != '\t')
-	    *p++ = '\n';
-	if (sp->Spooling && sp->SpoolName) {
-	    sprintf(p, "\tSpooling to \"%s\"\n", sp->SpoolName);
-	    p += strlen(p);
-	}
-	if ((cp = sp->Channel) != NULL) {
-	    sprintf(p, "\tChannel created %.12s",
-		ctime(&cp->Started) + 4);
-	    p += strlen(p);
-	    sprintf(p, ", last active %.12s\n",
-		ctime(&cp->LastActive) + 4);
-	    p += strlen(p);
-	    if (cp->Waketime > Now.time) {
-		sprintf(p, "\tSleeping until %.12s\n",
-		    ctime(&cp->Waketime) + 4);
-		p += strlen(p);
-	    }
+            buffer_append(bp, "\n", 1);
+	if (sp->Spooling && sp->SpoolName)
+            buffer_append_sprintf(bp, "\tSpooling to \"%s\"\n", sp->SpoolName);
+        cp = sp->Channel;
+	if (cp != NULL) {
+            buffer_append_sprintf(bp, "\tChannel created %.12s",
+                           ctime(&cp->Started) + 4);
+            buffer_append_sprintf(bp, ", last active %.12s\n",
+                           ctime(&cp->LastActive) + 4);
+	    if (cp->Waketime > Now.tv_sec)
+                buffer_append_sprintf(bp, "\tSleeping until %.12s\n",
+                               ctime(&cp->Waketime) + 4);
 	}
 
     }
-    buffer_append(bp, buff, p - buff);
+    buffer_append(bp, "", 1);
 }
