@@ -1,4 +1,4 @@
-/*  $Id: status.c 8781 2009-11-15 09:24:59Z iulius $
+/*  $Id: status.c 9206 2011-06-14 19:07:37Z iulius $
 **
 **  Periodic status reporting.
 */
@@ -13,12 +13,6 @@
 #include "innperl.h"
 
 #define MIN_REFRESH   60  /* 1 min */
-#define HTML_STATUS
-#if defined(HTML_STATUS)
-#define STATUS_FILE	"inn_status.html"	/* will be in pathhttp */
-#else
-#define STATUS_FILE	"inn.status"		/* will be in pathlog */
-#endif
 
 typedef struct _STATUS {
     char		name[SMBUF];
@@ -104,7 +98,7 @@ static void
 STATUSsummary(void)
 {
   FILE			*F;
-  int			i, j;
+  int			i;
   CHANNEL               *cp;
   int			activeCxn = 0;
   int			sleepingCxns = 0;
@@ -126,31 +120,30 @@ STATUSsummary(void)
   char			str[9];
   time_t		now;
  
-#if defined(HTML_STATUS)
-  path = concatpath(innconf->pathhttp, STATUS_FILE);
-#else
-  path = concatpath(innconf->pathlog, STATUS_FILE);
-#endif
+  if (innconf->htmlstatus) {
+    path = concatpath(innconf->pathhttp, "inn_status.html");
+  } else {
+    path = concatpath(innconf->pathlog, "inn.status");
+  }
   if ((F = Fopen(path, "w", TEMPORARYOPEN)) == NULL) {
     syswarn("SERVER cant open %s", path);
     return;
   }
 
-#if defined(HTML_STATUS)
-  /* HTML Header */
-
-  fprintf (F,"<HTML>\n<HEAD>\n<META HTTP-EQUIV=\"Refresh\" CONTENT=\"%lu;\">\n",
-	   innconf->status < MIN_REFRESH ? MIN_REFRESH : innconf->status);
-  fprintf (F, "<TITLE>%s: incoming feeds</TITLE>\n", innconf->pathhost);
-  fprintf (F, "</HEAD>\n<BODY>\n<PRE>\n") ;
-#endif /* defined(HTML_STATUS) */
+  /* HTML header. */
+  if (innconf->htmlstatus) {
+    fprintf(F, "<!DOCTYPE html>\n<html lang=\"en\">\n<head>\n");
+    fprintf(F, "<meta http-equiv=\"refresh\" content=\"%lu\">\n",
+            innconf->status < MIN_REFRESH ? MIN_REFRESH : innconf->status);
+    fprintf(F, "<title>%s: incoming feeds</title>\n", innconf->pathhost);
+    fprintf(F, "</head>\n<body>\n<pre>\n");
+  }
 
   fprintf (F, "%s\n", INN_VERSION_STRING);
   fprintf (F, "pid %d started %s\n", (int) getpid(), start_time);
 
   tmp = head = NULL;
   for (i = 0; (cp = CHANiter(&i, CTnntp)) != NULL; ) {
-    j = 0;
     strlcpy(TempString,
 	    cp->Address.ss_family == 0 ? "localhost" : RChostname(cp),
             sizeof(TempString));
@@ -385,10 +378,10 @@ STATUSsummary(void)
     status = tmp;
   }
 
-#if defined(HTML_STATUS)
-  /* HTML Footer */
-  fprintf (F,"</PRE>\n</BODY>\n</HTML>\n");
-#endif /* defined(HTML_STATUS) */
+  /* HTML footer. */
+  if (innconf->htmlstatus) {
+    fprintf (F, "</pre>\n</body>\n</html>\n");
+  }
 
   Fclose(F);
   free(path);
