@@ -1,4 +1,4 @@
-/*  $Id: endpoint.c 9275 2011-07-17 19:09:26Z iulius $
+/*  $Id: endpoint.c 9895 2015-06-14 10:08:21Z iulius $
 **
 **  The implementation of the innfeed EndPoint object class.
 **
@@ -16,7 +16,6 @@
 #include "config.h"
 #include "clibrary.h"
 #include "portable/socket.h"
-#include "portable/time.h"
 
 #include <assert.h>
 #include <errno.h>
@@ -34,6 +33,11 @@
 # include <sys/select.h>
 #endif
 
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
+
 #include "inn/innconf.h"
 #include "inn/messages.h"
 #include "inn/libinn.h"
@@ -317,7 +321,7 @@ void delEndPoint (EndPoint ep)
   /* Adjust the global arrays to account for deleted endpoint. */
   endPoints [ep->myFd] = NULL ;
   if (ep->myFd == highestFd)
-    while (endPoints [highestFd] == NULL && highestFd >= 0)
+    while (highestFd >= 0 && endPoints [highestFd] == NULL)
       highestFd-- ;
 
   for (idx = 0 ; idx < priorityCount ; idx++)
@@ -502,8 +506,8 @@ TimeoutId prepareWake (EndpTCB func, time_t timeToWake, void *clientData)
   id = timerElemAdd (timeToWake,func,clientData) ;
 
 #if 0
-  d_printf (1,"Preparing wake %d at date %ld for %d seconds\n",
-           (int) id, (long) now, timeToWake - now) ;
+  d_printf (1, "Preparing wake %d at date %ld for %ld seconds\n",
+           (int) id, (long) now, (long) (timeToWake - now)) ;
 #endif
 
   return id ;
@@ -519,8 +523,8 @@ TimeoutId prepareSleep (EndpTCB func, int timeToSleep, void *clientData)
   id = timerElemAdd (now + timeToSleep,func,clientData) ;
 
 #if 0
-  d_printf (1,"Preparing sleep %d at date %ld for %d seconds\n",
-           (int) id, (long) now, timeToSleep) ;
+  d_printf (1, "Preparing sleep %d at date %ld for %ld seconds\n",
+           (int) id, (long) now, (long) timeToSleep) ;
 #endif
 
   return id ;
@@ -940,8 +944,6 @@ static IoStatus doRead (EndPoint endp)
 
   bCount = (bCount > IOV_MAX ? IOV_MAX : bCount) ;
 
-  i = 0 ;
-
   /* now set up the iovecs for the readv */
   if (bCount > 0)
     {
@@ -1064,8 +1066,6 @@ static IoStatus doWrite (EndPoint endp)
 
   bCount = (bCount > IOV_MAX ? IOV_MAX : bCount) ;
 
-  i = 0 ;
-  
   if (bCount > 0)
     {
       vp = xcalloc (bCount, sizeof(struct iovec)) ;
