@@ -1,4 +1,4 @@
-/*  $Id: main.c 9479 2013-06-03 18:12:39Z iulius $
+/*  $Id: main.c 9926 2015-08-08 17:08:02Z iulius $
 **
 **  Main routines for the innfeed program.
 **
@@ -9,7 +9,6 @@
 #include "config.h"
 #include "clibrary.h"
 #include "portable/socket.h"
-#include "portable/time.h"
 
 #include <assert.h>
 #include <ctype.h>
@@ -21,8 +20,13 @@
 #include <sys/stat.h>
 #include <syslog.h>
 
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
+
 #if defined(HAVE_UNIX_DOMAIN_SOCKETS)
-# include <sys/un.h>
+# include "portable/socket-unix.h"
 #endif
 
 #include "inn/innconf.h"
@@ -52,7 +56,7 @@ const char *InputFile ;
 char *configFile = NULL ;
 bool RollInputFile = false ;
 char *pidFile = NULL ;
-bool useMMap = false ; /* Even if HAVE_MMAP.  Only used when innfeed is given
+bool useMMap = true ;  /* Even if !HAVE_MMAP.  Only used when innfeed is given
                         * file names to send instead of storage API tokens,
                         * which is a fairly rare use case. */
 void (*gPrintInfo) (void) ;
@@ -288,7 +292,7 @@ int main (int argc, char **argv)
 
   if ( !checkConfig ) 
     {
-      notice ("ME starting at %s\n (%s)", dateString, INN_VERSION_STRING);
+      notice("ME starting at %s (%s)", dateString, INN_VERSION_STRING);
     }
 
   val = true;
@@ -362,6 +366,7 @@ int main (int argc, char **argv)
   debugFile = concatpath(innconf->pathlog, DEBUG_FILE);
   if (loggingLevel == 0 && fileExistsP (debugFile))
     loggingLevel = 1 ;
+  free(debugFile);
 
   if (logFile == NULL && ! isatty (fileno (stderr)))
     logFile = concatpath(innconf->pathlog, LOG_FILE);
