@@ -1,4 +1,4 @@
-/* $Id: asprintf.c 9554 2013-11-03 19:18:12Z iulius $
+/* $Id: asprintf.c 9913 2015-07-07 16:31:52Z iulius $
  *
  * Replacement for a missing asprintf and vasprintf.
  *
@@ -8,7 +8,7 @@
  * The canonical version of this file is maintained in the rra-c-util package,
  * which can be found at <http://www.eyrie.org/~eagle/software/rra-c-util/>.
  *
- * Written by Russ Allbery <rra@stanford.edu>
+ * Written by Russ Allbery <eagle@eyrie.org>
  *
  * The authors hereby relinquish any claim to any copyright that they may have
  * in this work, whether granted under contract or by operation of law or
@@ -20,6 +20,7 @@
  */
 
 #include "config.h"
+#include "portable/macros.h"
 #include "clibrary.h"
 
 #include <errno.h>
@@ -29,11 +30,14 @@
  * with the system versions.
  */
 #if TESTING
+# undef asprintf
+# undef vasprintf
 # define asprintf test_asprintf
 # define vasprintf test_vasprintf
 int test_asprintf(char **, const char *, ...)
     __attribute__((__format__(printf, 2, 3)));
-int test_vasprintf(char **, const char *, va_list);
+int test_vasprintf(char **, const char *, va_list)
+    __attribute__((__format__(printf, 2, 0)));
 #endif
 
 
