@@ -1,4 +1,4 @@
-/*  $Id: conffile.c 7585 2006-11-21 09:37:51Z eagle $
+/*  $Id: conffile.c 9582 2013-12-09 21:30:42Z iulius $
 **
 **  Routines for reading in incoming.conf-style config files.
 */
@@ -41,7 +41,7 @@ static char *CONFgetword(CONFFILE *F)
   char *s;
   char *t;
   char *word;
-  bool flag, comment;
+  bool comment;
 
   if (!F) return (NULL);	/* No conf file */
   if (!F->buf || !F->buf[0]) {
@@ -58,15 +58,13 @@ static char *CONFgetword(CONFFILE *F)
      if ((p = strchr(F->buf, '\n')) != NULL)
        *p = '\0';
      for (p = F->buf; *p == ' ' || *p == '\t' ; p++);
-     flag = true;
      if ((*p == '\0' || *p == '#') && !cfeof(F)) {
-       flag = false;
        if (getconfline(F, F->buf, F->sbuf))
          return (NULL); /* Line too long */
        continue;
      }
      break;
-  } while (!cfeof(F) || !flag);
+  } while (!cfeof(F));
 
   comment = false;
   if (*p == '"') { /* double quoted string ? */
