@@ -1,4 +1,4 @@
-/*  $Id: dbz.c 9325 2011-08-06 20:55:43Z iulius $
+/*  $Id: dbz.c 9935 2015-09-02 12:23:29Z iulius $
 **
 **  dbz database implementation V6.1.1
 **
@@ -75,9 +75,13 @@
 #include <fcntl.h>
 #include <netinet/in.h>
 #include <sys/stat.h>
-#include <sys/time.h>
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
 
 #include "inn/dbz.h"
+#include "inn/fdflag.h"
 #include "inn/messages.h"
 #include "inn/innconf.h"
 #include "inn/mmap.h"
@@ -108,14 +112,11 @@ static int dbzversion = 6;	/* for validating .dir file format */
 #else
 #define of_t		off_t
 #endif
+
+#ifdef DO_TAGGED_HASH
 #define	SOF		(sizeof(of_t))
 #define	NOTFOUND	((of_t)-1)
-#ifdef	DO_TAGGED_HASH
-
-#define OVERFLOW
-#ifdef OVERFLOW
 #include <limits.h>
-#endif
 
 /* MAXDROPBITS is the maximum number of bits dropped from the offset value.
    The least significant bits are dropped.  The space is used to
@@ -129,7 +130,7 @@ static int dbzversion = 6;	/* for validating .dir file format */
 /*
  * We assume that unused areas of a binary file are zeros, and that the
  * bit pattern of `(of_t)0' is all zeros.  The alternative is rather
- * painful file initialization.  Note that okayvalue(), if OVERFLOW is
+ * painful file initialization.  Note that okayvalue(), if DO_TAGGED_HASH is
  * defined, knows what value of an offset would cause overflow.
  */
 #define VACANT		((of_t)0)
@@ -670,7 +671,7 @@ openhashtable(const char *base, const char *ext, hash_table *tab,
     free(name);
 
     tab->reclen = reclen;
-    close_on_exec(tab->fd, true);
+    fdflag_close_exec(tab->fd, true);
     tab->pos = -1;
 
     /* get first table into core, if it looks desirable and feasible */
@@ -685,7 +686,7 @@ openhashtable(const char *base, const char *ext, hash_table *tab,
 	}
     }
 
-    if (options.nonblock && nonblocking(tab->fd, true) < 0) {
+    if (options.nonblock && !fdflag_nonblocking(tab->fd, true)) {
 	syswarn("fcntl: could not set nonblock");
         oerrno = errno;
 	close(tab->fd);
@@ -721,7 +722,7 @@ openbasefile(const char *name)
     } else
 	basefname = NULL;
     if (basef != NULL)
-	close_on_exec(fileno(basef), true);
+	fdflag_close_exec(fileno(basef), true);
     if (basef != NULL)
 	 setvbuf(basef, NULL, _IOFBF, 64);
     return true;
@@ -758,7 +759,7 @@ dbzinit(const char *name)
 	syswarn("dbzinit: can't open .dir file");
 	return false;
     }
-    close_on_exec(fileno(dirf), true);
+    fdflag_close_exec(fileno(dirf), true);
 
     /* pick up configuration */
     if (!getconf(dirf, &conf)) {
@@ -883,11 +884,9 @@ static int
 okayvalue(of_t value)
 {
     if (HASTAG(value))
-	return(0);
-#ifdef OVERFLOW
+        return(0);
     if (value == LONG_MAX)	/* BIAS() and UNBIAS() will overflow */
-	return(0);
-#endif
+        return(0);
     return(1);
 }
 #endif
@@ -1148,7 +1147,8 @@ getconf(FILE *df, dbzconfig *cp)
 		&cp->valuesize, &cp->fillpercent, &cp->tagenb,
 		&cp->tagmask, &cp->tagshift, &cp->dropbits);
     if (i != 7) {
-	warn("dbz: bad first line in config");
+	warn("dbz: bad first line in .dir history file");
+    warn("dbz: you should consider running makedbz manually");
 	return false;
     }
     if (cp->valuesize != sizeof(of_t)) {
@@ -1170,7 +1170,7 @@ getconf(FILE *df, dbzconfig *cp)
     i = fscanf(df, "dbz 6 %ld %d %d\n", &cp->tsize,
 		&cp->valuesize, &cp->fillpercent);
     if (i != 3) {
-        warn("dbz: bad first line in config");
+        warn("dbz: bad first line in .dir history file");
 	return false;
     }
     if (cp->valuesize != (sizeof(of_t) + sizeof(erec))) {
@@ -1183,7 +1183,7 @@ getconf(FILE *df, dbzconfig *cp)
     /* second line, the usages */
     for (i = 0; i < NUSEDS; i++)
 	if (!fscanf(df, "%ld", &cp->used[i])) {
-            warn("dbz: bad usage value in config");
+            warn("dbz: bad usage value in .dir history file");
 	    return false;
 	}
     debug("used %ld %ld %ld...", cp->used[0], cp->used[1], cp->used[2]);
@@ -1192,7 +1192,7 @@ getconf(FILE *df, dbzconfig *cp)
     /* third line, the text usages */
     for (i = 0; i < NUSEDS; i++)
 	if (!fscanf(df, "%ld", &cp->vused[i])) {
-            warn("dbz: bad text usage value in config");
+            warn("dbz: bad text usage value in .dir history file");
 	    return false;
 	}
     debug("vused %ld %ld %ld...", cp->vused[0], cp->vused[1], cp->vused[2]);
@@ -1313,14 +1313,14 @@ putcore(hash_table *tab)
     if (tab->incore == INCORE_MEM) {
 	if(options.writethrough)
 	    return true;
-	nonblocking(tab->fd, false);
+	fdflag_nonblocking(tab->fd, false);
 	size = tab->reclen * conf.tsize;
         result = xpwrite(tab->fd, tab->core, size, 0);
 	if (result < 0 || (size_t) result != size) {
-	    nonblocking(tab->fd, options.nonblock);
+	    fdflag_nonblocking(tab->fd, options.nonblock);
 	    return false;
 	}
-	nonblocking(tab->fd, options.nonblock);
+	fdflag_nonblocking(tab->fd, options.nonblock);
     }
 #ifdef HAVE_MMAP
     if(tab->incore == INCORE_MMAP) {
@@ -1593,12 +1593,12 @@ set_pag(searcher *sp, of_t value)
     of_t v = value;
 
     if (CANTAG(v)) {
-	v |= sp->tag | taghere;
-	if (v != UNBIAS(VACANT))	/* BIAS(v) won't look VACANT */
-#ifdef OVERFLOW
-	    if (v != LONG_MAX)		/* and it won't overflow */
-#endif
-		value = v;
+        v |= sp->tag | taghere;
+        if (v != UNBIAS(VACANT)) {      /* BIAS(v) won't look VACANT */
+            if (v != LONG_MAX) {        /* and it won't overflow */
+                value = v;
+            }
+        }
     } else if (canttag_warned == 0) {
 	fprintf(stderr, "dbz.c(set): can't tag value 0x%lx", v);
 	fprintf(stderr, " tagboth = 0x%lx\n", tagboth);
