@@ -1,35 +1,71 @@
-/*  $Id: getaddrinfo.c 7400 2005-10-08 19:24:55Z eagle $
-**
-**  Replacement for a missing getaddrinfo.
-**
-**  Written by Russ Allbery <rra@stanford.edu>
-**  This work is hereby placed in the public domain by its author.
-**
-**  This is an implementation of getaddrinfo for systems that don't have one
-**  so that networking code can use a consistant interface without #ifdef.  It
-**  is a fairly minimal implementation, with the following limitations:
-**
-**    - IPv4 support only.  IPv6 is not supported.
-**    - AI_ADDRCONFIG is ignored.
-**    - Not thread-safe due to gethostbyname and getservbyname.
-**    - SOCK_DGRAM and SOCK_STREAM only.
-**    - Multiple possible socket types only generate one addrinfo struct.
-**    - Protocol hints aren't used correctly.
-**
-**  The last four issues could probably be easily remedied, but weren't
-**  needed for INN's purposes.  Adding IPv6 support isn't worth it; systems
-**  with IPv6 support should already support getaddrinfo natively.
-*/
+/* $Id: getaddrinfo.c 9883 2015-05-25 10:03:20Z iulius $
+ *
+ * Replacement for a missing getaddrinfo.
+ *
+ * This is an implementation of getaddrinfo for systems that don't have one so
+ * that networking code can use a consistant interface without #ifdef.  It is
+ * a fairly minimal implementation, with the following limitations:
+ *
+ *   - IPv4 support only.  IPv6 is not supported.
+ *   - AI_ADDRCONFIG is ignored.
+ *   - Not thread-safe due to gethostbyname and getservbyname.
+ *   - SOCK_DGRAM and SOCK_STREAM only.
+ *   - Multiple possible socket types only generate one addrinfo struct.
+ *   - Protocol hints aren't used correctly.
+ *
+ * The last four issues could probably be easily remedied, but haven't been
+ * needed to date.  Adding IPv6 support isn't worth it; systems with IPv6
+ * support should already support getaddrinfo natively.
+ *
+ * The canonical version of this file is maintained in the rra-c-util package,
+ * which can be found at <http://www.eyrie.org/~eagle/software/rra-c-util/>.
+ *
+ * Written by Russ Allbery <eagle@eyrie.org>
+ *
+ * The authors hereby relinquish any claim to any copyright that they may have
+ * in this work, whether granted under contract or by operation of law or
+ * international treaty, and hereby commit to the public, at large, that they
+ * shall not, at any time in the future, seek to enforce any copyright in this
+ * work against any person or entity, or prevent any person or entity from
+ * copying, publishing, distributing or creating derivative works of this
+ * work.
+ */
 
 #include "config.h"
 #include "clibrary.h"
 #include "portable/socket.h"
+
 #include <errno.h>
 
-/* If we're running the test suite, rename the functions to avoid conflicts
-   with the system version.  Note that we don't rename the structures and
-   constants, but that should be okay (except possibly for gai_strerror. */
+/* We need access to h_errno to map errors from gethostbyname. */
+#if !HAVE_DECL_H_ERRNO
+extern int h_errno;
+#endif
+
+/*
+ * The netdb constants, which aren't always defined (particularly if h_errno
+ * isn't declared).  We also make sure that a few of the less-used ones are
+ * defined so that we can deal with them in case statements.
+ */
+#ifndef HOST_NOT_FOUND
+# define HOST_NOT_FOUND 1
+# define TRY_AGAIN      2
+# define NO_RECOVERY    3
+# define NO_DATA        4
+#endif
+#ifndef NETDB_INTERNAL
+# define NETDB_INTERNAL -1
+#endif
+
+/*
+ * If we're running the test suite, rename the functions to avoid conflicts
+ * with the system version.  Note that we don't rename the structures and
+ * constants, but that should be okay (except possibly for gai_strerror).
+ */
 #if TESTING
+# undef gai_strerror
+# undef freeaddrinfo
+# undef getaddrinfo
 # define gai_strerror test_gai_strerror
 # define freeaddrinfo test_freeaddrinfo
 # define getaddrinfo  test_getaddrinfo
@@ -37,9 +73,13 @@ const char *test_gai_strerror(int);
 void test_freeaddrinfo(struct addrinfo *);
 int test_getaddrinfo(const char *, const char *, const struct addrinfo *,
                      struct addrinfo **);
+#endif
 
-/* If the native platform doesn't support AI_NUMERICSERV or AI_NUMERICHOST,
-   pick some other values for them. */
+/*
+ * If the native platform doesn't support AI_NUMERICSERV or AI_NUMERICHOST,
+ * pick some other values for them.
+ */
+#if TESTING
 # if AI_NUMERICSERV == 0
 #  undef AI_NUMERICSERV
 #  define AI_NUMERICSERV 0x0080
@@ -50,6 +90,20 @@ int test_getaddrinfo(const char *, const char *, const struct addrinfo *,
 # endif
 #endif
 
+/*
+ * Value representing all of the hint flags set.  Linux uses flags up to
+ * 0x0400, and Mac OS X up to 0x1000, so be sure not to break when testing
+ * on these platforms.
+ */
+#if TESTING
+# ifdef HAVE_GETADDRINFO
+#  define AI_INTERNAL_ALL 0x1fff
+# else
+#  define AI_INTERNAL_ALL 0x01ff
+# endif
+#else
+# define AI_INTERNAL_ALL 0x007f
+#endif
 
 /* Table of strings corresponding to the EAI_* error codes. */
 static const char * const gai_errors[] = {
@@ -65,30 +119,24 @@ static const char * const gai_errors[] = {
     "Supplied buffer too small",        /* 10 EAI_OVERFLOW */
 };
 
-/* Value representing all of the hint flags set.  Linux uses flags up to
-   0x0400, so be sure not to break when testing on that platform. */
-#if TESTING
-# ifdef HAVE_GETADDRINFO
-#  define AI_INTERNAL_ALL 0x04ff
-# else
-#  define AI_INTERNAL_ALL 0x01ff
-# endif
-#else
-# define AI_INTERNAL_ALL 0x007f
-#endif
-
 /* Macro to set the len attribute of sockaddr_in. */
 #if HAVE_STRUCT_SOCKADDR_SA_LEN
-# define sin_set_length(s)      ((s)->sin_len  = sizeof(struct sockaddr_in))
+# define sin_set_length(s) ((s)->sin_len  = sizeof(struct sockaddr_in))
 #else
-# define sin_set_length(s)      /* empty */
+# define sin_set_length(s) /* empty */
 #endif
 
+/*
+ * Used for iterating through arrays.  ARRAY_SIZE returns the number of
+ * elements in the array (useful for a < upper bound in a for loop).
+ */
+#define ARRAY_SIZE(array) (sizeof(array) / sizeof((array)[0]))
+
 
 /*
-**  Return a constant string for a given EAI_* error code or a string
-**  indicating an unknown error.
-*/
+ * Return a constant string for a given EAI_* error code or a string
+ * indicating an unknown error.
+ */
 const char *
 gai_strerror(int ecode)
 {
@@ -100,8 +148,8 @@ gai_strerror(int ecode)
 
 
 /*
-**  Free a linked list of addrinfo structs.
-*/
+ * Free a linked list of addrinfo structs.
+ */
 void
 freeaddrinfo(struct addrinfo *ai)
 {
@@ -109,10 +157,8 @@ freeaddrinfo(struct addrinfo *ai)
 
     while (ai != NULL) {
         next = ai->ai_next;
-        if (ai->ai_addr != NULL)
-            free(ai->ai_addr);
-        if (ai->ai_canonname != NULL)
-            free(ai->ai_canonname);
+        free(ai->ai_addr);
+        free(ai->ai_canonname);
         free(ai);
         ai = next;
     }
@@ -120,13 +166,13 @@ freeaddrinfo(struct addrinfo *ai)
 
 
 /*
-**  Convert a numeric service string to a number with error checking,
-**  returning true if the number was parsed correctly and false otherwise.
-**  Stores the converted number in the second argument.  Equivalent to calling
-**  strtol, but with the base always fixed at 10, with checking of errno,
-**  ensuring that all of the string is consumed, and checking that the
-**  resulting number is positive.
-*/
+ * Convert a numeric service string to a number with error checking, returning
+ * true if the number was parsed correctly and false otherwise.  Stores the
+ * converted number in the second argument.  Equivalent to calling strtol, but
+ * with the base always fixed at 10, with checking of errno, ensuring that all
+ * of the string is consumed, and checking that the resulting number is
+ * positive.
+ */
 static bool
 convert_service(const char *string, long *result)
 {
@@ -143,26 +189,28 @@ convert_service(const char *string, long *result)
 
 
 /*
-**  Allocate a new addrinfo struct, setting some defaults given that this
-**  implementation is IPv4 only.  Also allocates an attached sockaddr_in and
-**  zeroes it, per the requirement for getaddrinfo.  Takes the socktype,
-**  canonical name (which is copied if not NULL), address, and port.  Returns
-**  NULL on a memory allocation failure.
-*/
+ * Allocate a new addrinfo struct, setting some defaults given that this
+ * implementation is IPv4 only.  Also allocates an attached sockaddr_in and
+ * zeroes it, per the requirement for getaddrinfo.  Takes the socktype,
+ * canonical name (which is copied if not NULL), address, and port.  Returns
+ * NULL on a memory allocation failure.
+ */
 static struct addrinfo *
 gai_addrinfo_new(int socktype, const char *canonical, struct in_addr addr,
                  unsigned short port)
 {
     struct addrinfo *ai;
+    struct sockaddr_in *sin;
 
     ai = malloc(sizeof(*ai));
     if (ai == NULL)
         return NULL;
-    ai->ai_addr = malloc(sizeof(struct sockaddr_in));
-    if (ai->ai_addr == NULL) {
+    sin = calloc(1, sizeof(struct sockaddr_in));
+    if (sin == NULL) {
         free(ai);
         return NULL;
     }
+    ai->ai_addr = (struct sockaddr *) sin;
     ai->ai_next = NULL;
     if (canonical == NULL)
         ai->ai_canonname = NULL;
@@ -173,28 +221,27 @@ gai_addrinfo_new(int socktype, const char *canonical, struct in_addr addr,
             return NULL;
         }
     }
-    memset(ai->ai_addr, 0, sizeof(struct sockaddr_in));
     ai->ai_flags = 0;
     ai->ai_family = AF_INET;
     ai->ai_socktype = socktype;
     ai->ai_protocol = (socktype == SOCK_DGRAM) ? IPPROTO_UDP : IPPROTO_TCP;
     ai->ai_addrlen = sizeof(struct sockaddr_in);
-    ((struct sockaddr_in *) ai->ai_addr)->sin_family = AF_INET;
-    ((struct sockaddr_in *) ai->ai_addr)->sin_addr = addr;
-    ((struct sockaddr_in *) ai->ai_addr)->sin_port = htons(port);
-    sin_set_length((struct sockaddr_in *) ai->ai_addr);
+    sin->sin_family = AF_INET;
+    sin->sin_addr = addr;
+    sin->sin_port = htons(port);
+    sin_set_length(sin);
     return ai;
 }
 
 
 /*
-**  Look up a service.  Takes the service name (which may be numeric), the
-**  hint flags, a pointer to the socket type (used to determine whether TCP or
-**  UDP services are of interest and, if 0, is filled in with the result of
-**  getservbyname if the service was not numeric), and a pointer to the
-**  addrinfo struct to fill in.  Returns 0 on success or an EAI_* error on
-**  failure.
-*/
+ * Look up a service.  Takes the service name (which may be numeric), the hint
+ * flags, a pointer to the socket type (used to determine whether TCP or UDP
+ * services are of interest and, if 0, is filled in with the result of
+ * getservbyname if the service was not numeric), and a pointer to the
+ * addrinfo struct to fill in.  Returns 0 on success or an EAI_* error on
+ * failure.
+ */
 static int
 gai_service(const char *servname, int flags, int *type, unsigned short *port)
 {
@@ -214,9 +261,11 @@ gai_service(const char *servname, int flags, int *type, unsigned short *port)
         else
             protocol = NULL;
 
-        /* We really technically should be generating an addrinfo struct for
-           each possible protocol unless type is set, but this works well
-           enough for what I need this for. */
+        /*
+         * We really technically should be generating an addrinfo struct for
+         * each possible protocol unless type is set, but this works well
+         * enough for what I need this for.
+         */
         servent = getservbyname(servname, protocol);
         if (servent == NULL)
             return EAI_NONAME;
@@ -233,13 +282,13 @@ gai_service(const char *servname, int flags, int *type, unsigned short *port)
 
 
 /*
-**  Look up a host and fill in a linked list of addrinfo structs with the
-**  results, one per IP address of the returned host.  Takes the name or IP
-**  address of the host as a string, the lookup flags, the type of socket (to
-**  fill into the addrinfo structs), the port (likewise), and a pointer to
-**  where the head of the linked list should be put.  Returns 0 on success or
-**  the appropriate EAI_* error.
-*/
+ * Look up a host and fill in a linked list of addrinfo structs with the
+ * results, one per IP address of the returned host.  Takes the name or IP
+ * address of the host as a string, the lookup flags, the type of socket (to
+ * fill into the addrinfo structs), the port (likewise), and a pointer to
+ * where the head of the linked list should be put.  Returns 0 on success or
+ * the appropriate EAI_* error.
+ */
 static int
 gai_lookup(const char *nodename, int flags, int socktype, unsigned short port,
            struct addrinfo **res)
@@ -307,8 +356,8 @@ gai_lookup(const char *nodename, int flags, int socktype, unsigned short port,
 
 
 /*
-**  The actual getaddrinfo implementation.
-*/
+ * The actual getaddrinfo implementation.
+ */
 int
 getaddrinfo(const char *nodename, const char *servname,
             const struct addrinfo *hints, struct addrinfo **res)
@@ -340,10 +389,12 @@ getaddrinfo(const char *nodename, const char *servname,
         socktype = 0;
     }
 
-    /* See what we're doing.  If nodename is null, either AI_PASSIVE is set or
-       we're getting information for connecting to a service on the loopback
-       address.  Otherwise, we're getting information for connecting to a
-       remote system. */
+    /*
+     * See what we're doing.  If nodename is null, either AI_PASSIVE is set or
+     * we're getting information for connecting to a service on the loopback
+     * address.  Otherwise, we're getting information for connecting to a
+     * remote system.
+     */
     if (servname == NULL)
         port = 0;
     else {
@@ -356,7 +407,10 @@ getaddrinfo(const char *nodename, const char *servname,
     else {
         if (servname == NULL)
             return EAI_NONAME;
-        addr.s_addr = (flags & AI_PASSIVE) ? INADDR_ANY : INADDR_LOOPBACK;
+        if ((flags & AI_PASSIVE) == AI_PASSIVE)
+            addr.s_addr = INADDR_ANY;
+        else
+            addr.s_addr = htonl(0x7f000001UL);
         ai = gai_addrinfo_new(socktype, NULL, addr, port);
         if (ai == NULL)
             return EAI_MEMORY;
