@@ -1,19 +1,19 @@
-/*  $Revision: 1.5 $
+/*  $Id: getfqdn.c 7585 2006-11-21 09:37:51Z eagle $
 **
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
-#include <netdb.h>
+
+#include "config.h"
 #include "clibrary.h"
-#include "paths.h"
-#include "libinn.h"
+#include <netdb.h>
+
+#include "inn/libinn.h"
+#include "inn/paths.h"
 
 
 /*
 **  Get the fully-qualified domain name for this host.
 */
-char *GetFQDN(void)
+char *GetFQDN(char *domain)
 {
     static char		buff[SMBUF];
     struct hostent	*hp;
@@ -45,17 +45,18 @@ char *GetFQDN(void)
      * turn it on, but don't complain to me. */
     if (strchr(hp->h_name, '.') == NULL) {
 	/* Try to force DNS lookup if NIS/whatever gets in the way. */
-	(void)strncpy(temp, buff, sizeof(temp) - 1);
-	temp[sizeof(temp) - 1] = '\0';
-	(void)strncat(temp, ".", sizeof(temp) - strlen(temp) - 1);
+        strlcpy(temp, buff, sizeof(temp));
+        strlcat(temp, ".", sizeof(temp));
 	hp = gethostbyname(temp);
     }
 #endif	/* 0 */
 
     /* First, see if the main name is a FQDN.  It should be. */
     if (hp != NULL && strchr(hp->h_name, '.') != NULL) {
-	if (strlen(hp->h_name) < sizeof buff - 1)
-	    return strcpy(buff, hp->h_name);
+	if (strlen(hp->h_name) < sizeof buff - 1) {
+	    strlcpy(buff, hp->h_name, sizeof(buff));
+            return buff;
+        }
 	/* Doesn't fit; make sure we don't return bad data next time. */
 	buff[0] = '\0';
 	return hp->h_name;
@@ -66,19 +67,21 @@ char *GetFQDN(void)
 	while ((p = *ap++) != NULL)
 	    if (strchr(p, '.') != NULL) {
 		/* Deja-vous all over again. */
-		if (strlen(p) < sizeof buff - 1)
-		    return strcpy(buff, p);
+		if (strlen(p) < sizeof buff - 1) {
+		    strlcpy(buff, p, sizeof(buff));
+                    return buff;
+                }
 		buff[0] = '\0';
 		return p ;
 	    }
 
     /* Give up:  Get the domain config param and append it. */
-    if ((p = innconf->domain) == NULL || *p == '\0')
+    if ((p = domain) == NULL || *p == '\0')
 	return NULL;
     if (strlen(buff) + 1 + strlen(p) > sizeof buff - 1)
 	/* Doesn't fit. */
 	return NULL;
-    (void)strcat(buff, ".");
-    (void)strcat(buff, p);
+    strlcat(buff, ".", sizeof(buff));
+    strlcat(buff, p, sizeof(buff));
     return buff;
 }
