@@ -1,4 +1,4 @@
-/* $Id: inet_ntop.c 9557 2013-11-05 20:11:17Z iulius $
+/* $Id: inet_ntop.c 9759 2014-12-04 20:11:38Z iulius $
  *
  * Replacement for a missing inet_ntop.
  *
@@ -11,7 +11,7 @@
  * The canonical version of this file is maintained in the rra-c-util package,
  * which can be found at <http://www.eyrie.org/~eagle/software/rra-c-util/>.
  *
- * Written by Russ Allbery <rra@stanford.edu>
+ * Written by Russ Allbery <eagle@eyrie.org>
  *
  * The authors hereby relinquish any claim to any copyright that they may have
  * in this work, whether granted under contract or by operation of law or
@@ -43,6 +43,7 @@
  * the system version.
  */
 #if TESTING
+# undef inet_ntop
 # define inet_ntop test_inet_ntop
 const char *test_inet_ntop(int, const void *, char *, socklen_t);
 #endif
