@@ -1,4 +1,4 @@
-/*  $Id: innconf.c 9366 2011-08-22 08:51:14Z iulius $
+/*  $Id: innconf.c 9922 2015-07-14 16:43:55Z iulius $
 **
 **  Manage the global innconf struct.
 **
@@ -126,7 +126,7 @@ const struct config config_table[] = {
     { K(server),                  STRING  (NULL) },
     { K(sourceaddress),           STRING  (NULL) },
     { K(sourceaddress6),          STRING  (NULL) },
-    { K(timer),                   UNUMBER    (0) },
+    { K(timer),                   UNUMBER  (600) },
 
     { K(runasuser),               STRING  (RUNASUSER) },
     { K(runasgroup),              STRING  (RUNASGROUP) },
@@ -156,7 +156,7 @@ const struct config config_table[] = {
     { K(blockbackoff),            UNUMBER  (120) },
     { K(chaninacttime),           UNUMBER  (600) },
     { K(chanretrytime),           UNUMBER  (300) },
-    { K(datamovethreshold),       UNUMBER (8192) },
+    { K(datamovethreshold),       UNUMBER (16384) },
     { K(dontrejectfiltered),      BOOL   (false) },
     { K(hiscachesize),            UNUMBER  (256) },
     { K(htmlstatus),              BOOL    (true) },
@@ -168,7 +168,7 @@ const struct config config_table[] = {
     { K(logcancelcomm),           BOOL   (false) },
     { K(logipaddr),               BOOL    (true) },
     { K(logsitename),             BOOL    (true) },
-    { K(logstatus),               BOOL   (false) },
+    { K(logstatus),               BOOL    (true) },
     { K(logtrash),                BOOL    (true) },
     { K(maxartsize),              UNUMBER (1000000) },
     { K(maxconnections),          UNUMBER   (50) },
@@ -184,7 +184,7 @@ const struct config config_table[] = {
     { K(refusecybercancels),      BOOL   (false) },
     { K(remembertrash),           BOOL    (true) },
     { K(stathist),                STRING  (NULL) },
-    { K(status),                  UNUMBER    (0) },
+    { K(status),                  UNUMBER  (600) },
     { K(verifycancels),           BOOL   (false) },
     { K(verifygroups),            BOOL   (false) },
     { K(wanttrash),               BOOL   (false) },
@@ -193,8 +193,9 @@ const struct config config_table[] = {
     { K(xrefslave),               BOOL   (false) },
 
     /* The following settings are specific to nnrpd. */
-    { K(addnntppostingdate),      BOOL    (true) },
-    { K(addnntppostinghost),      BOOL    (true) },
+    { K(addinjectiondate),        BOOL    (true) },
+    { K(addinjectionpostingaccount), BOOL (false) },
+    { K(addinjectionpostinghost), BOOL    (true) },
     { K(allownewnews),            BOOL    (true) },
     { K(backoffauth),             BOOL   (false) },
     { K(backoffdb),               STRING  (NULL) },
@@ -212,7 +213,7 @@ const struct config config_table[] = {
     { K(keywords),                BOOL   (false) },
     { K(localmaxartsize),         UNUMBER (1000000) },
     { K(maxcmdreadsize),          UNUMBER (BUFSIZ) },
-    { K(msgidcachesize),          UNUMBER (16000) },
+    { K(msgidcachesize),          UNUMBER (64000) },
     { K(moderatormailer),         STRING  (NULL) },
     { K(nfsreader),               BOOL   (false) },
     { K(nfsreaderdelay),          UNUMBER   (60) },
@@ -221,17 +222,22 @@ const struct config config_table[] = {
     { K(nnrpdflags),              STRING    ("") },
     { K(nnrpdauthsender),         BOOL   (false) },
     { K(nnrpdloadlimit),          UNUMBER   (16) },
-    { K(nnrpdoverstats),          BOOL   (false) },
+    { K(nnrpdoverstats),          BOOL    (true) },
     { K(organization),            STRING  (NULL) },
     { K(readertrack),             BOOL   (false) },
     { K(spoolfirst),              BOOL   (false) },
     { K(strippostcc),             BOOL   (false) },
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
     { K(tlscafile),               STRING    ("") },
     { K(tlscapath),               STRING  (NULL) },
     { K(tlscertfile),             STRING  (NULL) },
     { K(tlskeyfile),              STRING  (NULL) },
-#endif /* HAVE_SSL */
+    { K(tlsciphers),              STRING  (NULL) },
+    { K(tlscompression),          BOOL    (true) },
+    { K(tlseccurve),              STRING  (NULL) },
+    { K(tlspreferserverciphers),  BOOL    (true) },
+    { K(tlsprotocols),            LIST    (NULL) },
+#endif /* HAVE_OPENSSL */
 
     /* The following settings are used by nnrpd and rnews. */
     { K(nnrpdposthost),           STRING  (NULL) },
@@ -244,12 +250,12 @@ const struct config config_table[] = {
     { K(keepmmappedthreshold),    UNUMBER (1024) },
     { K(nfswriter),               BOOL   (false) },
     { K(nnrpdcheckart),           BOOL    (true) },
-    { K(overcachesize),           UNUMBER   (64) },
+    { K(overcachesize),           UNUMBER  (128) },
     { K(ovgrouppat),              STRING  (NULL) },
     { K(storeonxref),             BOOL    (true) },
     { K(tradindexedmmap),         BOOL    (true) },
     { K(useoverchan),             BOOL   (false) },
-    { K(wireformat),              BOOL   (false) },
+    { K(wireformat),              BOOL    (true) },
 
     /* The following settings are specific to the history subsystem. */
     { K(hismethod),               STRING  (NULL) },
@@ -270,7 +276,11 @@ const struct config config_table[] = {
     { K(innwatchspoolnodes),      UNUMBER  (200) },
     { K(innwatchspoolspace),      UNUMBER (25000) },
 
-    /* The following settings are specific to scanlogs. */
+    /* The following settings are specific to scanlogs.
+     * Keep a low value by default as it has a privacy impact; three days
+     * should be enough to diagnose the most common cases of malfunction
+     * and abuse.  Where more is needed, it is easy for the news admin
+     * to increase it. */
     { K(logcycles),               UNUMBER    (3) },
 };
 
@@ -369,7 +379,7 @@ innconf_set_defaults(void)
         innconf->extraoverviewhidden = vector_new();
 
     /* Defaults used only if TLS (SSL) is supported. */
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
     if (innconf->tlscapath == NULL)
         innconf->tlscapath = xstrdup(innconf->pathetc);
     if (innconf->tlscertfile == NULL)
@@ -400,6 +410,7 @@ innconf_parse(struct config_group *group)
     struct innconf *config;
 
     config = xmalloc(sizeof(struct innconf));
+    memset(config, 0, sizeof(struct innconf));
     for (i = 0; i < ARRAY_SIZE(config_table); i++)
         switch (config_table[i].type) {
         case TYPE_BOOLEAN:
