@@ -1,8 +1,8 @@
-/*  $Id: memcmp.c 5049 2001-12-12 09:06:00Z rra $
+/*  $Id: memcmp.c 9767 2014-12-07 21:13:43Z iulius $
 **
 **  Replacement for a missing or broken memcmp.
 **
-**  Written by Russ Allbery <rra@stanford.edu>
+**  Written by Russ Allbery <eagle@eyrie.org>
 **  This work is hereby placed in the public domain by its author.
 **
 **  Provides the same functionality as the standard library routine memcmp
@@ -16,6 +16,7 @@
 /* If we're running the test suite, rename memcmp to avoid conflicts with
    the system version. */
 #if TESTING
+# undef memcmp
 # define memcmp test_memcmp
 int test_memcmp(const void *, const void *, size_t);
 #endif
