@@ -1,4 +1,4 @@
-/*  $Id: qio.c,v 1.8.2.3 2004/06/10 22:20:24 hkehoe Exp $
+/*  $Id: qio.c 9889 2015-06-03 19:05:15Z iulius $
 **
 **  Quick I/O package.
 **
@@ -15,22 +15,28 @@
 #include <sys/stat.h>
 
 #include "inn/qio.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 
 /* A reasonable default buffer size to use. */
 #define QIO_BUFFERSIZE  8192
 
+#ifdef HAVE_STRUCT_STAT_ST_BLKSIZE
+#define NO_STRUCT_STAT_ST_BLKSIZE_UNUSED
+#else
+#define NO_STRUCT_STAT_ST_BLKSIZE_UNUSED UNUSED
+#endif
+
 /*
 **  Given a file descriptor, return a reasonable buffer size to use for that
 **  file.  Uses st_blksize if available and reasonable, QIO_BUFFERSIZE
 **  otherwise.
 */
 static size_t
-buffer_size(int fd)
+buffer_size(int fd NO_STRUCT_STAT_ST_BLKSIZE_UNUSED)
 {
     size_t size = QIO_BUFFERSIZE;
 
-#if HAVE_ST_BLKSIZE
+#if HAVE_STRUCT_STAT_ST_BLKSIZE
     struct stat st;
 
     /* The Solaris 2.6 man page says that st_blksize is not defined for
@@ -44,7 +50,7 @@ buffer_size(int fd)
 	    while(size < QIO_BUFFERSIZE)
 		size += st.st_blksize;
     }
-#endif /* HAVE_ST_BLKSIZE */
+#endif /* HAVE_STRUCT_STAT_ST_BLKSIZE */
 
     return size;
 }
