@@ -1,4 +1,4 @@
-/*  $Id: remopen.c 8921 2010-01-22 23:33:08Z iulius $
+/*  $Id: remopen.c 9691 2014-09-17 16:31:35Z iulius $
 **
 **  Open a connection to a remote NNTP server.
 */
@@ -35,7 +35,7 @@ NNTPconnect(const char *host, int port, FILE **FromServerp, FILE **ToServerp,
     }
     *buff = '\0';
 
-    fd = network_connect_host(host, port, NULL);
+    fd = network_connect_host(host, port, NULL, DEFAULT_TIMEOUT);
     if (fd < 0)
         return -1;
 
