@@ -1,11 +1,11 @@
-/*  $Id: sendarticle.c 8016 2008-09-07 13:31:58Z iulius $
+/*  $Id: sendarticle.c 8903 2010-01-17 18:21:56Z iulius $
 **
 */
 
 #include "config.h"
 #include "clibrary.h"
 #include "inn/libinn.h"
-#include "nntp.h"
+#include "inn/nntp.h"
 
 
 /*
