@@ -1,4 +1,4 @@
-/*  $Id: timer.c 7585 2006-11-21 09:37:51Z eagle $
+/*  $Id: timer.c 9941 2015-09-05 18:03:04Z eagle $
 **
 **  Timer functions, to gather profiling data.
 **
@@ -39,9 +39,13 @@
 
 #include "config.h"
 #include "clibrary.h"
-#include "portable/time.h"
 #include <syslog.h>
 
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
+
 #include "inn/messages.h"
 #include "inn/timer.h"
 #include "inn/libinn.h"
@@ -110,6 +114,7 @@ static unsigned long
 TMRgettime(bool reset)
 {
     unsigned long now;
+    long usec, msec;
     struct timeval tv;
 
     /* The time of the last summary, used as a base for times returned by
@@ -121,8 +126,10 @@ TMRgettime(bool reset)
     static struct timeval base;
 
     gettimeofday(&tv, NULL);
-    now = (tv.tv_sec - base.tv_sec) * 1000;
-    now += (tv.tv_usec - base.tv_usec) / 1000;
+    now = (unsigned long)(tv.tv_sec - base.tv_sec) * 1000u;
+    usec = tv.tv_usec - base.tv_usec; /* maybe negative */
+    msec = usec / 1000;               /* still maybe negative */
+    now += (unsigned long)msec;
     if (reset)
         base = tv;
     return now;
@@ -366,10 +373,10 @@ TMRsummary(const char *prefix, const char *const *labels)
         off = 0;
     else
         off = snprintf(buf, len, "%s ", prefix);
-    off += snprintf(buf + off, len - off, "time %ld ", TMRgettime(true));
+    off += snprintf(buf + off, len - off, "time %lu ", TMRgettime(true));
     for (i = 0; i < timer_count; i++)
         if (timers[i] != NULL)
             off += TMRsumone(labels, timers[i], buf + off, len - off);
-    syslog(LOG_NOTICE, "%s", buf);
+    notice("%s", buf);
     free(buf);
 }
