@@ -1,4 +1,4 @@
-/*  $Id: article.c 9248 2011-07-17 18:04:50Z iulius $
+/*  $Id: article.c 9846 2015-05-03 14:56:15Z iulius $
 **
 **  Article-related routines.
 */
@@ -20,7 +20,7 @@
 #include "tls.h"
 #include "cache.h"
 
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
 extern SSL *tls_conn;
 #endif 
 
@@ -62,6 +62,8 @@ PushIOvHelper(struct iovec* vec, int* countp)
 {
     int result = 0;
 
+    TMRstart(TMR_NNTPWRITE);
+
 #ifdef HAVE_SASL
     if (sasl_conn && sasl_ssf) {
 	int i;
@@ -72,9 +74,7 @@ PushIOvHelper(struct iovec* vec, int* countp)
     } else {
 #endif /* HAVE_SASL */
 
-        TMRstart(TMR_NNTPWRITE);
-
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
 	if (tls_conn) {
 Again:
 	    result = SSL_writev(tls_conn, vec, *countp);
@@ -94,15 +94,15 @@ Again:
 		break;
 	    }
 	} else
-#endif /* HAVE_SSL */
+#endif /* HAVE_OPENSSL */
 	    result = xwritev(STDOUT_FILENO, vec, *countp);
 
-        TMRstop(TMR_NNTPWRITE);
-
 #ifdef HAVE_SASL
     }
 #endif
 
+    TMRstop(TMR_NNTPWRITE);
+
     if (result == -1) {
 	/* We can't recover, since we can't resynchronise with our
 	 * peer. */
@@ -241,7 +241,8 @@ ARTinstorebytoken(TOKEN token)
     if (PERMaccessconf->nnrpdoverstats) {
 	gettimeofday(&stv, NULL);
     }
-    art = SMretrieve(token, RETR_STAT); /* XXX This isn't really overstats, is it? */
+    art = SMretrieve(token, RETR_STAT);
+    /* XXX This isn't really overstats, is it? */
     if (PERMaccessconf->nnrpdoverstats) {
 	gettimeofday(&etv, NULL);
 	OVERartcheck+=(etv.tv_sec - stv.tv_sec) * 1000;
@@ -675,7 +676,6 @@ CMDfetch(int ac, char *av[])
                   PERMcanauthenticate ? NNTP_FAIL_AUTH_NEEDED : NNTP_ERR_ACCESS);
 	    return;
 	}
-	tart=art;
 	Reply("%d %lu %s %s\r\n", what->ReplyCode, art, av[1], what->Item);
 	if (what->Type != STstat) {
 	    ARTsendmmap(what->Type);
@@ -872,14 +872,14 @@ vhost_xref(char *p)
 
     space = strchr(p, ' ');
     if (space == NULL) {
-	warn("malformed Xref: `%s'", field);
-	goto fail;
+        warn("malformed Xref: `%s'", p);
+        goto fail;
     }
     offset = space - p;
     space = strchr(p + offset, ' ');
     if (space == NULL) {
-	warn("malformed Xref: `%s'", field);
-	goto fail;
+        warn("malformed Xref: `%s'", p);
+        goto fail;
     }
     field = concat(PERMaccessconf->domain, space, NULL);
  fail:
