@@ -1,21 +1,22 @@
-/*  $Id: commands.c 8923 2010-01-22 23:35:12Z iulius $
+/*  $Id: commands.c 9850 2015-05-03 15:25:53Z iulius $
 **
 **  Miscellaneous commands.
 */
 #include "config.h"
 #include "clibrary.h"
-#include "portable/wait.h"
+#include <sys/wait.h>
 
 #include "nnrpd.h"
+#include "inn/fdflag.h"
 #include "inn/ov.h"
 #include "inn/innconf.h"
 #include "inn/messages.h"
 #include "inn/version.h"
 #include "tls.h"
 
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
 extern bool nnrpd_starttls_done;
-#endif /* HAVE_SSL */
+#endif /* HAVE_OPENSSL */
 
 typedef struct {
     char *name;
@@ -87,8 +88,8 @@ makeGroupInvalid(void) {
 
 /*  Returns:
 **    -1 for problem (such as no such authenticator, etc.).
-**     0 for authentication succeeded.
-**     1 for authentication failed.
+**     1 for authentication succeeded.
+**     0 for authentication failed.
 */
 static char *PERMauthstring;
 
@@ -96,6 +97,7 @@ static int
 PERMgeneric(char *av[], char *accesslist, size_t size)
 {
     char path[BIG_BUFFER], *fields[6], *p;
+    size_t j;
     int i, pan[2], status;
     pid_t pid;
     struct stat stb;
@@ -183,9 +185,9 @@ PERMgeneric(char *av[], char *accesslist, size_t size)
 	    close(pan[PIPE_WRITE]);
 	}
 
-	close_on_exec(STDIN_FILENO, false);
-	close_on_exec(STDOUT_FILENO, false);
-	close_on_exec(STDERR_FILENO, false);
+	fdflag_close_exec(STDIN_FILENO, false);
+	fdflag_close_exec(STDOUT_FILENO, false);
+	fdflag_close_exec(STDERR_FILENO, false);
 
 	execv(path, av);
         /* RFC 2980 requires 500 if there are unspecified errors during
@@ -197,11 +199,11 @@ PERMgeneric(char *av[], char *accesslist, size_t size)
     }
 
     close(pan[PIPE_WRITE]);
-    i = read(pan[PIPE_READ], path, sizeof(path));
+    read(pan[PIPE_READ], path, sizeof(path));
 
     waitpid(pid, &status, 0);
     if (!WIFEXITED(status) || WEXITSTATUS(status) != 0)
-        return 1;
+        return 0;
 
     if ((p = strchr(path, '\n')) != NULL)
 	*p = '\0';
@@ -213,12 +215,25 @@ PERMgeneric(char *av[], char *accesslist, size_t size)
 
     //syslog(L_NOTICE, "%s (%ld) returned: %d %s %d\n", av[0], (long) pid, i, path, status);
     /* Split "host:permissions:user:pass:groups" into fields. */
-    for (fields[0] = path, i = 0, p = path; *p; p++)
-	if (*p == ':') {
-	    *p = '\0';
-	    fields[++i] = p + 1;
+    for (fields[0] = path, j = 0, p = path; *p; p++)
+        if (*p == ':') {
+            *p = '\0';
+            ++j;
+            if (j < ARRAY_SIZE(fields)) {
+                fields[j] = p + 1;
+            } else {
+                Reply("%d Program error occurred\r\n", NNTP_FAIL_ACTION);
+                syslog(L_FATAL, "over-long response from %s", av[0]);
+                return -1;
+            }
 	}
 
+    if (j < 4) {
+        Reply("%d Program error occurred\r\n", NNTP_FAIL_ACTION);
+        syslog(L_FATAL, "short response from %s", av[0]);
+        return -1;
+    }
+
     PERMcanread = strchr(fields[1], 'R') != NULL;
     PERMcanpost = strchr(fields[1], 'P') != NULL;
     PERMaccessconf->allowapproved = strchr(fields[1], 'A') != NULL;
@@ -232,7 +247,7 @@ PERMgeneric(char *av[], char *accesslist, size_t size)
     //for (i = 0; fields[i] && i < 5; i++)
     //    syslog(L_NOTICE, "fields[%d] = %s\n", i, fields[i]);
 
-    return 0;
+    return 1;
 }
 
 
@@ -254,7 +269,7 @@ CMDauthinfo(int ac, char *av[])
 
         /* Go on parsing the command line. */
         ac--;
-        ac += reArgify(av[ac], &av[ac], -1, true);
+        (void) reArgify(av[ac], &av[ac], -1, true);
 
 	strlcpy(PERMuser, "<none>", sizeof(PERMuser));
 
@@ -308,7 +323,7 @@ CMDauthinfo(int ac, char *av[])
                 return;
             }
 
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
             /* Check whether STARTTLS must be used before trying to authenticate. */
             if (PERMcanauthenticate && !PERMcanauthenticatewithoutSSL
                 && !nnrpd_starttls_done) {
@@ -337,7 +352,7 @@ CMDauthinfo(int ac, char *av[])
             return;
         }
 
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
         /* Check whether STARTTLS must be used before trying to authenticate. */
         if (PERMcanauthenticate && !PERMcanauthenticatewithoutSSL
             && !nnrpd_starttls_done) {
@@ -616,7 +631,7 @@ CMDpost(int ac, char *av[])
     }
     if (!ihave && !PERMcanpost) {
 	syslog(L_NOTICE, "%s noperm post without permission", Client.host);
-        Reply("%d Posting not allowed\r\n",
+	Reply("%d Posting not allowed\r\n",
               PERMcanauthenticate && PERMcanpostgreeting ?
               NNTP_FAIL_AUTH_NEEDED : NNTP_FAIL_POST_AUTH);
 	return;
@@ -777,7 +792,7 @@ CMDpost(int ac, char *av[])
     }
 
     /* Send the article to the server. */
-    response = ARTpost(article, idbuff, ihave, &permanent);
+    response = ARTpost(article, idbuff, &permanent);
     if (response == NULL) {
         notice("%s %s ok %s", Client.host, ihave ? "ihave" : "post", idbuff);
         Reply("%d Article received %s\r\n", ihave ? NNTP_OK_IHAVE : NNTP_OK_POST, idbuff);
