@@ -1,4 +1,4 @@
-/*  $Id: misc.c 9259 2011-07-17 18:22:50Z iulius $
+/*  $Id: misc.c 9563 2013-11-10 13:13:34Z iulius $
 **
 **  Miscellaneous support routines.
 */
@@ -15,10 +15,10 @@
 #include "nnrpd.h"
 #include "tls.h"
 
-/* Outside the ifdef so that make depend works even ifndef HAVE_SSL. */
+/* Outside the ifdef so that make depend works even ifndef HAVE_OPENSSL. */
 #include "inn/ov.h"
 
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
 extern SSL *tls_conn;
 extern int tls_cipher_usebits;
 extern char *tls_peer_CN;
@@ -335,6 +335,7 @@ GetPostRecord(char *path, long *lastpost, long *lastsleep, long *lastn)
      if (fgets(buff,SMBUF,fp) == NULL) {
        syslog(L_ERROR, "%s Error reading '%s': %s",
               Client.host, path, strerror(errno));
+       fclose(fp);
        return 0;
      }
      *lastpost = atol(buff);
@@ -342,6 +343,7 @@ GetPostRecord(char *path, long *lastpost, long *lastsleep, long *lastn)
      if ((s = strchr(buff,',')) == NULL) {
        syslog(L_ERROR, "%s bad data in postrec file: '%s'",
               Client.host, buff);
+       fclose(fp);
        return 0;
      }
      s++; *lastsleep = atol(s);
@@ -349,6 +351,7 @@ GetPostRecord(char *path, long *lastpost, long *lastsleep, long *lastn)
      if ((s = strchr(s,',')) == NULL) {
        syslog(L_ERROR, "%s bad data in postrec file: '%s'",
               Client.host, buff);
+       fclose(fp);
        return 0;
      }
      s++; *lastn = atol(s);
@@ -446,7 +449,7 @@ RateLimit(long *sleeptime, char *path)
      return 1;
 }
 
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
 /*
 **  The STARTTLS command.  RFC 4642.
 */
@@ -522,4 +525,4 @@ CMDstarttls(int ac UNUSED, char *av[] UNUSED)
     /* Reset our read buffer so as to prevent plaintext command injection. */
     line_reset(&NNTPline);
 }
-#endif /* HAVE_SSL */
+#endif /* HAVE_OPENSSL */
