@@ -1,11 +1,11 @@
-/*  $Id: nnrpd.h 9259 2011-07-17 18:22:50Z iulius $
+/*  $Id: nnrpd.h 9795 2015-03-18 20:31:16Z iulius $
 **
 **  NetNews Reading Protocol server.
 */
 
 #include "config.h"
+#include "portable/macros.h"
 #include "portable/socket.h"
-#include "portable/time.h"
 
 #include <ctype.h>
 #include <errno.h>
@@ -13,6 +13,11 @@
 #include <syslog.h>
 #include <sys/stat.h>
 
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
+
 #include "inn/qio.h"
 #include "inn/libinn.h"
 #include "inn/nntp.h"
@@ -21,6 +26,9 @@
 #include "inn/vector.h"
 #include "inn/timer.h"
 
+#ifndef HAVE_SASL_SASL_H
+# undef HAVE_SASL
+#endif
 #ifdef HAVE_SASL
 #include <sasl/sasl.h>
 #include <sasl/saslutil.h>
@@ -65,8 +73,9 @@ typedef struct _ACCESSGROUP {
     unsigned long localmaxartsize;
     int readertrack;
     int strippostcc;
-    int addnntppostinghost;
-    int addnntppostingdate;
+    int addinjectiondate;
+    int addinjectionpostingaccount;
+    int addinjectionpostinghost;
     char *nnrpdposthost;
     unsigned long nnrpdpostport;
     int nnrpdoverstats;
@@ -151,7 +160,7 @@ enum timer {
 
 EXTERN bool	PERMauthorized;
 EXTERN bool     PERMcanauthenticate;
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
 EXTERN bool     PERMcanauthenticatewithoutSSL;
 #endif
 EXTERN bool	PERMcanpost;
@@ -208,8 +217,7 @@ EXTERN struct vector *OVextra;
 EXTERN int	overhdr_xref;
 EXTERN bool     LLOGenable;
 
-extern const char	*ARTpost(char *article, char *idbuff, bool ihave,
-				 bool *permanent);
+extern const char	*ARTpost(char *article, char *idbuff, bool *permanent);
 extern void		ARTclose(void);
 extern int		TrimSpaces(char *line);
 extern void		InitBackoffConstants(void);
@@ -218,7 +226,8 @@ extern int		LockPostRec(char *path);
 extern int		LockPostRec(char *path);
 extern void		UnlockPostRec(char *path);
 extern int		RateLimit(long *sleeptime, char *path);
-extern void		ExitWithStats(int x, bool readconf) __attribute__ ((noreturn));
+extern void		ExitWithStats(int x, bool readconf)
+    __attribute__ ((__noreturn__));
 extern char		*GetHeader(const char *header, bool stripspaces);
 extern void		GRPreport(void);
 extern bool		NGgetlist(char ***argvp, char *list);
@@ -247,11 +256,12 @@ extern void             CMDnewnews      (int ac, char** av);
 extern void             CMDnextlast     (int ac, char** av);
 extern void             CMDover         (int ac, char** av);
 extern void             CMDpost         (int ac, char** av);
-extern void             CMDquit         (int ac, char** av) __attribute__ ((noreturn));
+extern void             CMDquit         (int ac, char** av)
+    __attribute__ ((__noreturn__));
 extern void             CMDxgtitle      (int ac, char** av);
 extern void             CMDpat          (int ac, char** av);
 extern void             CMD_unimp       (int ac, char** av);
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
 extern void             CMDstarttls     (int ac, char** av);
 #endif
 
