@@ -1,4 +1,4 @@
-/*  $Id: perm.c 9603 2014-02-17 13:18:21Z iulius $
+/*  $Id: perm.c 9847 2015-05-03 15:01:03Z iulius $
 **
 **  How to figure out where a user comes from, and what that user can do once
 **  we know who sie is.
@@ -6,9 +6,9 @@
 
 #include "config.h"
 #include "clibrary.h"
-#include "portable/wait.h"
 #include <netdb.h>
 #include <signal.h>
+#include <sys/wait.h>
 
 #include "conffile.h"
 #include "inn/network.h"
@@ -21,9 +21,9 @@
 # include <sys/select.h>
 #endif
 
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
 extern bool nnrpd_starttls_done;
-#endif /* HAVE_SSL */
+#endif /* HAVE_OPENSSL */
 
 /* Data types. */
 typedef struct _CONFCHAIN {
@@ -42,7 +42,7 @@ typedef struct _METHOD {
 typedef struct _AUTHGROUP {
     char *name;
     char *key;
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
     int require_ssl;
 #endif
     char *hosts;
@@ -137,34 +137,35 @@ extern bool PerlLoaded;
 #define PERMlocalmaxartsize	35
 #define PERMreadertrack		36
 #define PERMstrippostcc		37
-#define PERMaddnntppostinghost	38
-#define PERMaddnntppostingdate	39
-#define PERMnnrpdposthost	40
-#define PERMnnrpdpostport	41
-#define PERMnnrpdoverstats	42
-#define PERMbackoff_auth	43
-#define PERMbackoff_db		44
-#define PERMbackoff_k		45
-#define PERMbackoff_postfast	46
-#define PERMbackoff_postslow	47
-#define PERMbackoff_trigger	48
-#define PERMnnrpdcheckart	49
-#define PERMnnrpdauthsender	50
-#define PERMvirtualhost		51
-#define PERMnewsmaster		52
-#define PERMlocaladdress	53
-#define PERMrejectwith		54
-#define PERMmaxbytespersecond	55
-#define PERMperl_auth           56
-#define PERMpython_auth         57
-#define PERMperl_access         58
-#define PERMpython_access       59
-#define PERMpython_dynamic      60
-#ifdef HAVE_SSL
-#define PERMrequire_ssl                61
-#define PERMMAX                        62
+#define PERMaddinjectiondate	38
+#define PERMaddinjectionpostingaccount 39
+#define PERMaddinjectionpostinghost	40
+#define PERMnnrpdposthost	41
+#define PERMnnrpdpostport	42
+#define PERMnnrpdoverstats	43
+#define PERMbackoff_auth	44
+#define PERMbackoff_db		45
+#define PERMbackoff_k		46
+#define PERMbackoff_postfast	47
+#define PERMbackoff_postslow	48
+#define PERMbackoff_trigger	49
+#define PERMnnrpdcheckart	50
+#define PERMnnrpdauthsender	51
+#define PERMvirtualhost		52
+#define PERMnewsmaster		53
+#define PERMlocaladdress	54
+#define PERMrejectwith		55
+#define PERMmaxbytespersecond	56
+#define PERMperl_auth           57
+#define PERMpython_auth         58
+#define PERMperl_access         59
+#define PERMpython_access       60
+#define PERMpython_dynamic      61
+#ifdef HAVE_OPENSSL
+#define PERMrequire_ssl         62
+#define PERMMAX                 63
 #else
-#define PERMMAX			61
+#define PERMMAX			62
 #endif
 
 #define TEST_CONFIG(a, b) \
@@ -226,8 +227,9 @@ static CONFTOKEN PERMtoks[] = {
     { PERMlocalmaxartsize,      (char *) "localmaxartsize:"     },
     { PERMreadertrack,          (char *) "readertrack:"         },
     { PERMstrippostcc,          (char *) "strippostcc:"         },
-    { PERMaddnntppostinghost,   (char *) "addnntppostinghost:"  },
-    { PERMaddnntppostingdate,   (char *) "addnntppostingdate:"  },
+    { PERMaddinjectiondate,     (char *) "addinjectiondate:"    },
+    { PERMaddinjectionpostingaccount, (char *) "addinjectionpostingaccount:" },
+    { PERMaddinjectionpostinghost, (char *) "addinjectionpostinghost:" },
     { PERMnnrpdposthost,        (char *) "nnrpdposthost:"       },
     { PERMnnrpdpostport,        (char *) "nnrpdpostport:"       },
     { PERMnnrpdoverstats,       (char *) "nnrpdoverstats:"      },
@@ -249,7 +251,7 @@ static CONFTOKEN PERMtoks[] = {
     { PERMperl_access,          (char *) "perl_access:"         },
     { PERMpython_access,        (char *) "python_access:"       },
     { PERMpython_dynamic,       (char *) "python_dynamic:"      },
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
     { PERMrequire_ssl,          (char *) "require_ssl:"         },
 #endif
     { 0,                        (char *) NULL                   }
@@ -349,7 +351,7 @@ copy_authgroup(AUTHGROUP *orig)
     else
 	ret->hosts = 0;
 
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
     ret->require_ssl = orig->require_ssl;
 #endif
 
@@ -451,7 +453,7 @@ copy_accessgroup(ACCESSGROUP *orig)
 static void
 SetDefaultAuth(AUTHGROUP *curauth UNUSED)
 {
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
         curauth->require_ssl = false;
 #endif
 }
@@ -491,8 +493,9 @@ SetDefaultAccess(ACCESSGROUP *curaccess)
     curaccess->localmaxartsize = innconf->localmaxartsize;
     curaccess->readertrack = innconf->readertrack;
     curaccess->strippostcc = innconf->strippostcc;
-    curaccess->addnntppostinghost = innconf->addnntppostinghost;
-    curaccess->addnntppostingdate = innconf->addnntppostingdate;
+    curaccess->addinjectiondate = innconf->addinjectiondate;
+    curaccess->addinjectionpostingaccount = innconf->addinjectionpostingaccount;
+    curaccess->addinjectionpostinghost = innconf->addinjectionpostinghost;
     curaccess->nnrpdposthost = innconf->nnrpdposthost;
     curaccess->nnrpdpostport = innconf->nnrpdpostport;
     curaccess->nnrpdoverstats = innconf->nnrpdoverstats;
@@ -630,7 +633,7 @@ static void
 authdecl_parse(AUTHGROUP *curauth, CONFFILE *f, CONFTOKEN *tok)
 {
     int oldtype;
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
     int boolval;
 #endif
     METHOD *m;
@@ -652,7 +655,7 @@ authdecl_parse(AUTHGROUP *curauth, CONFFILE *f, CONFTOKEN *tok)
 	ReportError(f, buff);
     }
 
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
     if (strcasecmp(tok->name, "on") == 0
         || strcasecmp(tok->name, "true") == 0
         || strcasecmp(tok->name, "yes") == 0)
@@ -670,7 +673,7 @@ authdecl_parse(AUTHGROUP *curauth, CONFFILE *f, CONFTOKEN *tok)
 	curauth->key = xstrdup(tok->name);
 	SET_CONFIG(PERMkey);
 	break;
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
       case PERMrequire_ssl:
         if (boolval != -1)
             curauth->require_ssl = boolval;
@@ -982,12 +985,19 @@ accessdecl_parse(ACCESSGROUP *curaccess, CONFFILE *f, CONFTOKEN *tok)
 	if (boolval != -1) curaccess->strippostcc = boolval;
 	SET_CONFIG(oldtype);
 	break;
-      case PERMaddnntppostinghost:
-	if (boolval != -1) curaccess->addnntppostinghost = boolval;
+      case PERMaddinjectiondate:
+	if (boolval != -1)
+            curaccess->addinjectiondate = boolval;
 	SET_CONFIG(oldtype);
 	break;
-      case PERMaddnntppostingdate:
-	if (boolval != -1) curaccess->addnntppostingdate = boolval;
+      case PERMaddinjectionpostingaccount:
+        if (boolval != -1)
+            curaccess->addinjectionpostingaccount = boolval;
+        SET_CONFIG(oldtype);
+        break;
+      case PERMaddinjectionpostinghost:
+	if (boolval != -1)
+            curaccess->addinjectionpostinghost = boolval;
 	SET_CONFIG(oldtype);
 	break;
       case PERMnnrpdposthost:
@@ -1125,7 +1135,7 @@ PERMreadfile(char *filename)
     /* Are we editing an auth or access group? */
 
     inwhat	= 0;
-    newgroup	= curgroup = 0;
+    curgroup = 0;
 
     tok		= CONFgettoken(PERMtoks, cf->f);
 
@@ -1256,7 +1266,7 @@ PERMreadfile(char *filename)
 
 		/* Stuff that belongs to an auth group. */
 	      case PERMhost:
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
               case PERMrequire_ssl:
 #endif
 	      case PERMauthprog:
@@ -1298,8 +1308,9 @@ PERMreadfile(char *filename)
 	      case PERMlocalmaxartsize:
 	      case PERMreadertrack:
 	      case PERMstrippostcc:
-	      case PERMaddnntppostinghost:
-	      case PERMaddnntppostingdate:
+	      case PERMaddinjectiondate:
+              case PERMaddinjectionpostingaccount:
+	      case PERMaddinjectionpostinghost:
 	      case PERMnnrpdposthost:
 	      case PERMnnrpdpostport:
 	      case PERMnnrpdoverstats:
@@ -1405,7 +1416,7 @@ PERMgetaccess(char *nnrpaccess)
 #else
     PERMcanauthenticate = false;
 #endif
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
     PERMcanauthenticatewithoutSSL = false;
 #endif
     PERMgroupmadeinvalid = false;
@@ -1438,7 +1449,7 @@ PERMgetaccess(char *nnrpaccess)
     for (i = 0; auth_realms[i]; i++) {
 	if (auth_realms[i]->auth_methods != NULL) {
 	    PERMcanauthenticate = true;
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
             if (auth_realms[i]->require_ssl == false)
                 PERMcanauthenticatewithoutSSL = true;
 #endif
@@ -1452,7 +1463,7 @@ PERMgetaccess(char *nnrpaccess)
     }
     uname = 0;
     while (!uname && i--) {
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
         /* If SSL is required, check that the connection is encrypted. */
         if ((auth_realms[i]->require_ssl == true) && !nnrpd_starttls_done)
             continue;
@@ -1700,7 +1711,7 @@ PERMgetpermissions(void)
     if (i >= 0) {
 	/* Found the right access group. */
 	if (access_realms[i]->rejectwith) {
-	    syslog(L_ERROR, "%s rejected by rule (%s)",
+	    syslog(L_NOTICE, "%s rejected by rule (%s)",
 		Client.host, access_realms[i]->rejectwith);
 	    Reply("%d Permission denied:  %s\r\n",
 		NNTP_FAIL_TERMINATING, access_realms[i]->rejectwith);
@@ -1878,6 +1889,7 @@ strip_accessgroups(void)
 	}
     } else {
 	syslog(L_TRACE, "No access realms to check!");
+        return;
     }
 
     /* If there are auth realms to check... */
@@ -1941,7 +1953,7 @@ ResolveUser(AUTHGROUP *auth)
     if (auth->res_methods == NULL)
         return NULL;
 
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
     /* If SSL is required, check that the connection is encrypted. */
     if ((auth->require_ssl == true) && !nnrpd_starttls_done)
         return NULL;
@@ -1991,7 +2003,7 @@ AuthenticateUser(AUTHGROUP *auth, char *username, char *password,
     if (auth->auth_methods == NULL)
         return NULL;
 
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
     /* If SSL is required, check that the connection is encrypted. */
     if ((auth->require_ssl == true) && !nnrpd_starttls_done)
         return NULL;
