@@ -1,4 +1,4 @@
-##  $Id: filter_innd.py 8640 2009-09-29 20:13:55Z iulius $
+##  $Id: filter_innd.py 8955 2010-02-07 20:05:39Z iulius $
 ##
 ##  This is a sample filter for the Python innd hook.
 ##
@@ -33,9 +33,12 @@ if sys.version_info[0] >= 3:
 ##  losing some strcmps under the covers.
 Also_Control = intern("Also-Control")
 Approved = intern("Approved")
+Archive = intern("Archive")
+Archived_At = intern("Archived-At")
 Bytes = intern("Bytes")
 Cancel_Key = intern("Cancel-Key")
 Cancel_Lock = intern("Cancel-Lock")
+Comments = intern("Comments")
 Content_Base = intern("Content-Base")
 Content_Disposition = intern("Content-Disposition")
 Content_Transfer_Encoding = intern("Content-Transfer-Encoding")
@@ -72,6 +75,7 @@ Relay_Version = intern("Relay-Version")
 Reply_To = intern("Reply-To")
 Sender = intern("Sender")
 Subject = intern("Subject")
+Summary = intern("Summary")
 Supersedes = intern("Supersedes")
 User_Agent = intern("User-Agent")
 X_Auth = intern("X-Auth")
@@ -103,7 +107,7 @@ X_Usenet_Provider = intern("X-Usenet-Provider")
 X_User_ID = intern("X-User-ID")
 Xref = intern("Xref")
 __BODY__ = intern("__BODY__")
-_LINES__ = intern("__LINES__")
+__LINES__ = intern("__LINES__")
 
 
 class InndFilter:
@@ -142,15 +146,13 @@ class InndFilter:
     def filter_messageid(self, msgid):
         """Filter articles just by their Message-IDs.
 
-        This method interacts with the IHAVE and CHECK NNTP commands.
+        This method interacts with the CHECK, IHAVE and TAKETHIS
+        NNTP commands.
         If you return a non-empty string here, the offered article
         will be refused before you ever have to waste any bandwidth
-        looking at it (make sure that such a message is properly
-        encoded in UTF-8 so as to comply with the NNTP protocol).
-        This is not foolproof, so you should do your ID checks both
-        here and in filter_art.  (TAKETHIS does not offer the ID
-        for examination, and a TAKETHIS isn't always preceded
-        by a CHECK.)
+        looking at it (unless TAKETHIS is used before an earlier CHECK).
+        Make sure that such a message is properly encoded in UTF-8
+        so as to comply with the NNTP protocol.
         """
         return ""               # Deactivate the samples.
 
@@ -169,7 +171,7 @@ class InndFilter:
         The available headers are the ones listed near the top of
         innd/art.c.  At this writing, they are:
 
-            Also-Control, Approved, Bytes, Cancel-Key, Cancel-Lock,
+            Also-Control, Approved, Archive, Archived-At, Bytes, Cancel-Key, Cancel-Lock,
             Content-Base, Content-Disposition, Content-Transfer-Encoding,
             Content-Type, Control, Date, Date-Received, Distribution, Expires,
             Face, Followup-To, From, In-Reply-To, Injection-Date, Injection-Info,
@@ -177,7 +179,7 @@ class InndFilter:
             NNTP-Posting-Date, NNTP-Posting-Host, NNTP-Posting-Path,
             Organization, Original-Sender, Originator,
             Path, Posted, Posting-Version, Received, References, Relay-Version,
-            Reply-To, Sender, Subject, Supersedes, User-Agent,
+            Reply-To, Sender, Subject, Summary, Supersedes, User-Agent,
             X-Auth, X-Auth-Sender, X-Canceled-By, X-Cancelled-By, X-Complaints-To,
             X-Face, X-HTTP-UserAgent, X-HTTP-Via, X-Mailer, X-Modbot, X-Modtrace,
             X-Newsposter, X-Newsreader, X-No-Archive, X-Original-Message-ID,
