@@ -1,4 +1,4 @@
-/*  $Id: expire.c 9024 2010-03-21 16:49:30Z iulius $
+/*  $Id: expire.c 9896 2015-06-14 10:09:47Z iulius $
 **
 **  Code for overview-driven expiration.
 **
@@ -157,12 +157,6 @@ EXPsplit(char *p, char sep, char **argv, int count)
     if (!p)
       return 0;
 
-    while (*p == sep)
-      ++p;
-
-    if (!*p)
-      return 0;
-
     for (i = 1, *argv++ = p; *p; )
         if (*p++ == sep) {
             p[-1] = '\0';
@@ -545,7 +539,7 @@ static void
 ARTreadschema(void)
 {
     const struct cvector        *standardoverview;
-    const struct vector         *extraoverview;
+    struct vector               *extraoverview;
     ARTOVERFIELD                *fp;
     unsigned int                i;
 
@@ -572,6 +566,7 @@ ARTreadschema(void)
     }
 
     ARTfieldsize = fp - ARTfields;
+    vector_free(extraoverview);
 }
 
 /*
@@ -764,7 +759,7 @@ OVgroupbasedexpire(TOKEN token, const char *group, const char *data,
             keeper = true;
         if ((krps[i] == Remove) && strcmp(group, arts[i]) == 0)
             delete = true;
-        if ((krps[i] == Keep))
+        if (krps[i] == Keep)
             purge = false;
     }
     EXPprocessed++;
