@@ -1,4 +1,4 @@
-/* $Id: confparse-t.c 8781 2009-11-15 09:24:59Z iulius $ */
+/* $Id: confparse-t.c 9764 2014-12-07 20:54:31Z iulius $ */
 /* confparse test suite. */
 
 #include "config.h"
@@ -8,7 +8,9 @@
 #include "inn/messages.h"
 #include "inn/vector.h"
 #include "inn/libinn.h"
-#include "libtest.h"
+#include "tap/basic.h"
+#include "tap/float.h"
+#include "tap/messages.h"
 
 /* Given a FILE *, read from that file, putting the results into a newly
    allocated buffer, until encountering a line consisting solely of "===".
@@ -513,7 +515,7 @@ main(void)
         exit(1);
     vector = config_params(group);
     ok_int(67, 2, vector->count);
-    ok_int(68, 2, vector->allocated);
+    skip(68, "test to be removed when using the new C TAP Harness syntax");
     if (strcmp(vector->strings[0], "foo") == 0)
         ok_string(69, "bar", vector->strings[1]);
     else if (strcmp(vector->strings[0], "bar") == 0)
@@ -636,7 +638,7 @@ main(void)
     vector = config_params(subgroup);
     ok(153, vector != NULL);
     ok_int(154, 3, vector->count);
-    ok_int(155, 3, vector->allocated);
+    skip(155, "test to be removed when using the new C TAP Harness syntax");
     ok_string(156, "third", vector->strings[0]);
     ok_string(157, "second", vector->strings[1]);
     ok_string(158, "first", vector->strings[2]);
@@ -691,7 +693,7 @@ main(void)
     n = test_warnings_uint(n);
     n = test_warnings_real(n);
     n = test_warnings_string(n);
-    n = test_warnings_list(n);
+    test_warnings_list(n);
 
     return 0;
 }
