@@ -1,4 +1,4 @@
-/* $Id: date-t.c 9280 2011-07-17 19:36:24Z iulius $ */
+/* $Id: date-t.c 9669 2014-09-03 17:18:22Z iulius $ */
 /* makedate test suite */
 
 #include "config.h"
@@ -6,7 +6,7 @@
 #include <time.h>
 
 #include "inn/libinn.h"
-#include "libtest.h"
+#include "tap/basic.h"
 
 static const time_t test_times[] = {
     28800UL,                    /* Thu,  1 Jan 1970 00:00:00 -0800 (PST) */
