@@ -1,4 +1,4 @@
-/* $Id: api-t.c 9280 2011-07-17 19:36:24Z iulius $ */
+/* $Id: api-t.c 9908 2015-06-25 20:00:51Z iulius $ */
 /* Test suite for overview API. */
 
 #include "config.h"
@@ -13,7 +13,7 @@
 #include "inn/overview.h"
 #include "inn/vector.h"
 #include "inn/libinn.h"
-#include "libtest.h"
+#include "tap/basic.h"
 #include "inn/storage.h"
 
 /* Used as the artificial token for all articles inserted into overview. */
@@ -74,6 +74,7 @@ fake_innconf(void)
         free(innconf);
     }
     innconf = xmalloc(sizeof(*innconf));
+    memset(innconf, 0, sizeof(*innconf));
     innconf->enableoverview = true;
     innconf->groupbaseexpiry = true;
     innconf->icdsynccount = 10;
@@ -294,7 +295,7 @@ check_data(const char *group, ARTNUM artnum, const char *expected,
     }
     if (data->arrived != (time_t) artnum * 10) {
         warn("Arrival time wrong for %s:%lu: %lu != %lu", group, artnum,
-             data->arrived, artnum * 10);
+             (unsigned long) data->arrived, artnum * 10);
         status = false;
     }
 
@@ -304,7 +305,7 @@ check_data(const char *group, ARTNUM artnum, const char *expected,
     expires = (artnum % 5 == 0) ? artnum * 100 : artnum;
     if (data->expires != 0) {
         warn("Expires time wrong for %s:%lu: %lu != %lu", group, artnum,
-             data->expires, 0UL);
+             (unsigned long) data->expires, 0UL);
         status = false;
     }
     return status;
@@ -704,7 +705,7 @@ main(void)
     free(innconf->ovmethod);
     innconf->ovmethod = xstrdup("buffindexed");
     diag("buffindexed");
-    n = overview_tests(n);
+    overview_tests(n);
 
     return 0;
 }
