@@ -1,4 +1,4 @@
-/* $Id: overview-t.c 8964 2010-02-08 20:55:59Z iulius $ */
+/* $Id: overview-t.c 9908 2015-06-25 20:00:51Z iulius $ */
 /* Overview test suite, usable for any of the overview methods. */
 
 /* Compile this program with OVTYPE defined to one of the valid overview
@@ -16,7 +16,7 @@
 #include "inn/messages.h"
 #include "inn/vector.h"
 #include "inn/libinn.h"
-#include "libtest.h"
+#include "tap/basic.h"
 #include "inn/ov.h"
 #include "inn/storage.h"
 
@@ -86,6 +86,7 @@ fake_innconf(void)
         free(innconf);
     }
     innconf = xmalloc(sizeof(*innconf));
+    memset(innconf, 0, sizeof(*innconf));
     innconf->enableoverview = true;
     innconf->groupbaseexpiry = true;
     innconf->icdsynccount = 10;
